/*! \file ALDApplication.h
 * \brief Объявление базового класса приложения Astra Linux Directory
 * \ingroup CALDApplication_group
 */

#ifndef ALD_APP
#define ALD_APP

#include "common.h"
#include "ALDCommand.h"
#include "ALDCore.h"
#include "ALDCommonCommands.h"

using namespace std;

namespace ALD
{

/// Результат разбора командной строки
enum ALDParseCommandLineResult {
    apclrSimple,    ///< Простой вызов
    apclrCommand,   ///< Команда
    apclrExit,      ///< Все отработано => выход
    apclrError      ///< Ошиба
};

/// Результат разбора общих опций
enum ALDCommonOptionResult {
    acorMiss,       ///< Не найдено
    acorHit,        ///< Отработано
    acorExit        ///< Требуется выход
};

/*! \defgroup CALDApplication_group Базовый класс приложения клиента (CALDApplication)
 * \brief Базовый класс приложения клиента Astra Linux Directory
 * @{
 */

/*! \class CALDApplication
 * \brief Базововый класс приложения клиента Astra Linux Directory
 */
class CALDApplication
{
protected:
    int m_Argc;                       ///< Число аргументов командной строки
    char** m_Argv;                    ///< Массив аргументов командной строки
    string m_strTxtDomain;            ///< Текстовый домен для локализации
    string m_strLogFile;              ///< Путь к файлу журнала
    string m_strDesc;                 ///< Наименование
    string m_strName;                 ///< Описание
    bool m_bSimple;                   ///< Признак простого запуска
    bool m_bRecheckServer;            ///< Признак поиска сервера Astra Linux Directory
    CALDCommandPtr CurrCmd;           ///< Текущая исполняемая команда
    CALDCommandOptions m_CLOptions;   ///< Текущие опции командной строки
    IALDCore* m_pCore;                ///< Указатель на интерфейс ядра Astra Linux Directory
    CALDCmdHelp *m_pHelp;             ///< Указатель на общую команду Help
    CALDCmdCmd *m_pCmd;               ///< Указатель на общую команду Cmd
    CALDCmdStatus *m_pStatus;         ///< Указатель на общую команду Status

    /// Разбор командной строки
    ALDParseCommandLineResult ParseCommandLine();

    /// Перегружаемая функции исполнения
    virtual int Exec();
    /// Перегружаемая функции простого исполнения
    virtual int SimpleExec();
    /// Перегружаемая функция инициализации
    virtual int Init() = 0;

    /// Перегружаемая функции вывода информации о способе использования
    virtual void ShowUsage();
    /// Перегружаемая функции вывода информации о версии
    virtual void ShowVersionInfo();

    /// Перегружаемая функции проверки параметров
    virtual void TestParm();

    /// Перегружаемая функции обработки опций командной строки
    virtual ALDCommonOptionResult ProcessCommonOption(const string& option);

    /// Перегружаемая функции загрузки параметров
    virtual void LoadSettings() = 0;

    /// Перегружаемая обработки командной строки
    virtual void OnParseCommandLine(){}

    /// Указания использования встроенных общих команд
    /// @{
    void UseHelpCommand();
    void UseCmdCommand();
    void UseStatusCommand();
    /// @}
public:
    CALDApplication();

    /// Запуск приложения с аргументами коммандной строки
    int Run(int Argc, char **Argv);

    /// Возвращает имя приложения (при старте)
    const string name(bool start) const;

    /// Установка заголовка процесса
    void SetProcTitle(const string& title);

    virtual ~CALDApplication();
};

///@}

} // ALD

#endif
