/*! \file ALDCommand.h
 * \brief Объявление классов командного интерфейса Astra Linux Directory
 * \ingroup ALDCommands_group
 */

#ifndef ALD_COMMAND
#define ALD_COMMAND

#include "common.h"
#include "ALDCore.h"
#include <map>
#include <list>
#include <memory>

using namespace std;

namespace ALD
{

/*! \defgroup ALDCommands_group Командный интерфейс Astra Linux Directory
 * \brief Командный интерфейс Astra Linux Directory
 * @{
 */

/// Тип необходимости аргумента
enum acArgumentNecessity {
    anNoArgument = 0,       ///< Аргумент не требуется
    anRequiredArgument = 1, ///< Аргумент обязателен
    anOptionalArgument = 2  ///< Аргумент может быть
};

/// Ожидаемый тип аргумента
enum acArgumentType {
    acatText,       ///< Текст
    acatNumber,     ///< Десятичное число
    acatHex,        ///< Шестнадцатиричное число
    acatOct,        ///< Восьмеричное число
    acatTPeriod     ///< Период времени
};

/*! \class CALDCommandOption
 * \brief Базовый класс опции команды
 */
class CALDCommandOption
{
protected:
    bool m_bActive;                       ///< Признак активности
    bool m_bMultiple;                     ///< Признак множественности
    bool m_bMandatory;                    ///< Признак необходимости
    string m_strName;                     ///< Имя опции
    int m_iShortIdent;                    ///< Короткое значение
    acArgumentNecessity m_HasArgument;    ///< Тип необходимости аргумента
    acArgumentType  m_ArgumentType;       ///< Ожидаемый тип аргумента
    ald_string_list m_lstArguments;       ///< Список аргументов
public:
    /// Возвращает имя
    inline const string &name() const {return m_strName;}
    /// Возвращает короткое значение
    inline int ident() const {return m_iShortIdent;}
    /// Возвращает признак активности
    inline bool IsActive() const {return m_bActive;}
    /// Возвращает признак множественности
    inline bool multiple() const {return m_bMultiple;}
    /// Возвращает признак необходимости
    inline bool mandatory() const {return m_bMandatory;}
    /// Возвращает тип необходимости аргумента
    inline acArgumentNecessity hasArgument() const {return m_HasArgument;}
    /// Возвращает ожидаемый тип аргумента
    inline acArgumentType argumentType() const {return m_ArgumentType;}
    /// Устанавливает активность безаргументной опции
    void SetActive();
    /// Устанавливает активность опции с указанием аргумента
    void SetActive(const string& arg);
    /// Возвращает текстовое представление аргумента
    string argument() const;
    /// Возвращает целое представление аргумента
    unsigned int uargument() const;
    /// Возвращает список аргументов
    inline ald_string_list& Arguments() {return m_lstArguments;}
    /// Очистка состояния опции
    virtual void Clear();
    /// Конструктор
    CALDCommandOption(const string& strName, int iShortIdent = 0,
                      acArgumentNecessity HasArgument = anNoArgument,
                      acArgumentType ArgumentType = acatText,
                      bool bMultiple = false, bool bMandatory = false);
    virtual ~CALDCommandOption();
};

/// Разделяемый указатель на опцию
typedef shared_ptr<CALDCommandOption> CALDCommandOptionPtr;

/// Карта опций (имя=>разделяемый указатель на опцию)
typedef map<string, CALDCommandOptionPtr> ald_command_option_map;

/*! \class CALDCommandOptions
 * \brief Класс контейнера опций команды
 */
class CALDCommandOptions
{
protected:
    ald_command_option_map m_mapOptions;    ///< Карта опций
    ald_command_option_map::iterator m_itOption;    ///< Итератор опций
public:
    CALDCommandOptions();
    /// Установка итератора на начало
    void begin();
    /// Продвижение итератора с возвратом текущей опции
    CALDCommandOptionPtr next();
    /// Возвращает опцию по имени
    CALDCommandOptionPtr OptionByName(const string& strName);
    /// Возвращает активность опци по имени
    bool IsActive(const string& opt);
    /// Добавляет опцию
    void Add(CALDCommandOptionPtr Option);
    /// Добавляет опцию
    void Add(CALDCommandOption* pOption);
    /// Удаляет опцию по имени
    void Remove(const string& strName);
    /// Заполнение контейнера опций содержимым другого контейнера опции
    void Assign(const CALDCommandOptions& Options);
    /// Копирование опций из другого контейнера опций
    void CopyFrom(CALDCommandOptions& Options);
    /// Очистка контейнера опций
    void Clear();
    /// Очистка состояния опций контейнера
    void ClearOptionsState();
    /// Размер контейнера (количество опций)
    inline size_t Size() const {return m_mapOptions.size();}
    virtual ~CALDCommandOptions();
};

/*! \class CALDCommand
 * \brief Абстрактный базовый класс команды
 */
class CALDCommand
{
protected:
    string m_strSection;                ///< Секция помощи
    string m_strName;                   ///< Имя
    string m_strDesc;                   ///< Описание
    CALDCommandOptions m_Options;       ///< Опции команды
    ald_string_list m_lstArguments;     ///< Список аргументов
    acArgumentNecessity m_HasArgument;  ///< Тип необходимости аргумента
    IALDCore* m_pCore;                  ///< Указатель на интерфейс ядра Astra Linux Directory
    bool m_bPrivileged;                 ///< Признак привилегированной команды
    bool m_bIndependent;                ///< Признак независимой команды

    bool CheckArguments();              ///< Проверка аргументов по типу необходимости

    virtual int Execute() = 0;          ///< Перегружаемая рабочая функция команды
public:
    /// Возвращает секцию команды
    inline const string &section() const {return m_strSection;}
    /// Возвращает имя
    inline const string &name() const {return m_strName;}
    /// Возвращает описание
    inline const string &desc() const {return m_strDesc;}
    /// Возвращает текстовое представление аргумента
    string argument() const;
    /// Возвращает тип необходимости аргумента
    inline acArgumentNecessity hasArgument() const {return m_HasArgument;}
    /// Возвращает признак привилегированной команды
    inline bool IsPrivileged() const {return m_bPrivileged;}
    /// Возвращает признак независимой команды
    inline bool IsIndependent() const {return m_bIndependent;}
    /// Перегружаемая функция описания использования
    virtual void ShowUsage() const = 0;
    /// Перегружаемая функция выполнения команды
    virtual int Run();
    /// Возвращает контейнер опций
    inline CALDCommandOptions& Options() {return m_Options;}
    /// Возвращает активность опции команды по имени
    inline bool OptionIsActive(const string& opt) { return m_Options.IsActive(opt);}
    /// Возвращет опцию по имени
    inline CALDCommandOptionPtr OptionByName(const string& strName) {return m_Options.OptionByName(strName);}
    /// Добавляет аргумент
    void AddArgument(const string& arg);
    /// Перегружаемая функция очистки аргументов и состояния опций
    virtual void Clear();
    /// Перегружаемая функция заполнения аргументов в ReadLine
    virtual void FillRLArguments(const string& option, ald_string_list& list, bool& isfile);
    /// Перегружаемая функция заполнения опций в ReadLine
    virtual void FillRLOptions(ald_string_list& list);
    /// Конструктор
    CALDCommand(IALDCore& Core);
    virtual ~CALDCommand();
};

/// Разделяемый указатель на команду
typedef shared_ptr<CALDCommand> CALDCommandPtr;
/// Карта команд (имя=>разделяемый указатель на команду
typedef map<string, CALDCommandPtr> ald_command_map;

/*! \class CALDCommands
 * \brief Класс контейнера команд
 */
class CALDCommands
{
protected:
    ald_command_map m_mapCommands;  ///< Карта команд
    ald_command_map::iterator m_itCommand; ///< Итератор команд
public:
    CALDCommands();
    /// Установка итератора на начало
    void begin();
    /// Продвижение итератора с возвратом текущей команды
    CALDCommandPtr next();
    /// Возвращает команду по имени
    CALDCommandPtr CommandByName(const string& strName);
    /// Удаляет команду по имени
    void Remove(const string& strName);
    /// Удаляет команды перечисленные в другом контейнере команд
    void RemoveCommands(CALDCommands& Commands);
    /// Добавляет команду
    void Add(CALDCommandPtr Command);
    /// Добавляет команду
    void Add(CALDCommand* pCommand);
    /// Добавляет команду
    void Add(CALDCommands& Commands);
    /// Заполнение контейнера команд содержимым другого контейнера команд
    void Assign(const CALDCommands& Commands);
    /// Очистка контейнера
    void Clear();
    /// Размер контейнера (количество команд)
    size_t Size() const;
    virtual ~CALDCommands();
};

///@}

} // ALD

#endif // ALD_COMMAND
