/*! \file ALDCore.h
 * \brief Объявление основного интерфейса клиента Astra Linux Directory и необходимых для него типов и интерфейсных классов
 * \ingroup IALDCore_group
 */

#ifndef ALD_CORE
#define ALD_CORE

#include "common.h"
#include "ALDCommand.h"
#include "ALDLogProvider.h"
#include <map>
#include <memory>

using namespace std;

/*! \namespace ALD
 * \brief Пространство имен основного интерфейса Astra Linux Directory
 */
namespace ALD
{
/*! \defgroup IALDCore_group Интерфейс ядра Astra Linux Directory (IALDCore)
 * \brief Центральный интерфейс Astra Linux Directory
 * @{
 */

/// Набор способов аутентификации
enum ALDAuthType {
    aatSimple,      ///< Анонимное соединение без аутентификации (только чтение LDAP)
    aatLocal,       ///< Локальная несетевая аутентификация для серверных компонент (без запроса пароля)
    aatGSS,         ///< Парольная аутентификация Kerberos
    aatGSSKeytab,   ///< Аутентификация Kerberos с использованием файла ключа (без запроса пароля)
    aatUser         ///< Аутентификация Kerberos с использованием текущего кеша (без запроса пароля)
};

/// Структура информации о модуле
struct ald_module_info {
    // cppcheck-suppress unusedStructMember
    int        api_version; ///< Версия API
    const char *name;       ///< Имя
    const char *version;    ///< Версия
    // cppcheck-suppress unusedStructMember
    int        type;        ///< Тип
    const char *desc;       ///< Описание
    // cppcheck-suppress unusedStructMember
    void       *extra;      ///< Эктра параметр
};

/// Версия API ALD
#define ALD_API_VERSION          1*0x10000+7*0x100+37

/// Версия модулей ALD
#define ALD_MODULE_VERSION      "1.7.37"

/*! \name Типы модулей */
///@{
#define ALD_MODULE_TYPE_HOST     0
#define ALD_MODULE_TYPE_ITF      1
#define ALD_MODULE_TYPE_CFG      2
#define ALD_MODULE_TYPE_PLUGIN   3
///@}

/// Тип функции получения информации о модуле
typedef ald_module_info* (ald_get_module_info_ft)(const ald_module_info &host);

/*! \name Константы режимов ядра */
///@{
#define ALD_MODE_UNDEFINED        ""
#define ALD_MODE_CLIENT           "client"
#define ALD_MODE_SERVER           "server"
#define ALD_MODE_ADMIN            "admin"
///@}

/// Тип функции инициализации модуля с указанием режима
typedef bool (ald_module_init_ft)(ALD::IALDCore &Core, const string &mode);

/// Тип функции освобождения модуля
typedef void (ald_module_done_ft)(ALD::IALDCore &Core);

/*! \class CALDInterface
 * \brief Базовый класс интерфейса
 */
class CALDInterface
{
protected:
    string m_strName;       ///< Наименование
    IALDCore* m_pCore;      ///< Указатель на интерфейс ядра
public:
    /// Возвращает наименование
    inline const string &name() const {return m_strName;}
    /// Конструктор интерфейса
    CALDInterface(const string &Name, IALDCore &Core):m_strName(Name),m_pCore(&Core){}
    virtual ~CALDInterface(){}
};

/// Разделяемый указатель на интерфейс
typedef shared_ptr<CALDInterface> CALDInterfacePtr;

/// Карта интерфейсов (имя => разделяемый указатель на интерфейс)
typedef map<string, CALDInterfacePtr> ald_interface_map;

/*! \name Оперции по конфигурированию */
///@{
#define ALD_OPR_CFG_INSTALL   "install"
#define ALD_OPR_CFG_CONFIGURE "configure"
#define ALD_OPR_CFG_ON        "on"
#define ALD_OPR_CFG_OFF       "off"
#define ALD_OPR_CFG_UNINSTALL "uninstall"
///@}

/*! \class IALDConfigurator
 * \brief Базовый класс конфигуратора
 */class IALDConfigurator: public CALDInterface
{
public:
    /// Конструктор конфигуратора
    IALDConfigurator(const string &Name, IALDCore &Core):CALDInterface(Name, Core){}
    /// Выполнение указанной для режима операции
    virtual int OperationConfig(const string &mode, const string &operation) = 0;
};

/// Разделяемый указатель на конфигуратор
typedef shared_ptr<IALDConfigurator> CALDConfiguratorPtr;

/// Карта конфигураторов (имя => разделяемый указатель на конфигуратор)
typedef map<string, CALDConfiguratorPtr> ald_configurator_map;

/*! \class CALDValidator
 * \brief Базовый класс валидатора
 */
class CALDValidator
{
protected:
    string m_strName;   ///< Имя
    IALDCore* m_pCore;  ///< Указатель на интерфейс Core
    string m_strError;  ///< Сообщение об ошибке
    void ClearError();  ///< Очистка ошибки
public:
    /// Возвращает имя
    inline const string &name() const {return m_strName;}
    /// Возвращает сообщение об ошибке
    inline const string &error() const {return m_strError;}

    /*! \brief Конструктор
     * @param[in] Name Имя
     * @param[in] Core Интерфейс Core
     */
    CALDValidator(const string &Name, IALDCore &Core):m_strName(Name),m_pCore(&Core){}

    /*! \brief Перегружаемый вызов валидации
     * @param[in] value Проверяемое значение
     * @param[in] tag Произвольная дополнительная информации
     * @param[out] result Результат (нормализованное значение)
     */
    virtual bool Validate(const void *value, const void *tag, void *result);

    /// Деструктор
    virtual ~CALDValidator(){}
};

/// Разделяемый указатель на валидатор
typedef shared_ptr<CALDValidator> CALDValidatorPtr;

/// Карта валидаторов (имя => разделяемый указатель на валидатор)
typedef map<string, CALDValidatorPtr> ald_validator_map;

/// Режим SigInt
enum asmSigIntMode {
    asmSigIntOn,    ///< Включен
    asmSigIntOff    ///< Выключен
};

/*! \brief Тип триггерной функции
* @param[in] Core Указатель на интерфейс ядра
* @param[in] action Событие
* @param[in] object Имя объекта
* @param[in] cmd Указатель на текущую команду
* @param[in] extra Доп. параметр
* @return Успех или отказ
*/
typedef bool ald_action_trigger_func(IALDCore* Core, const string &action, const string &object,
                                     CALDCommand *cmd, void* extra);
/// Структура записи о триггере
struct ald_action_trigger {
    string name;                       ///< Имя события
    // cppcheck-suppress unusedStructMember
    ald_action_trigger_func* trigger;  ///< Указатель на триггерную функцию
};

/// Структура события ALD_TEST_INTEGRITY
struct ald_test_integrity_extra {
    // cppcheck-suppress unusedStructMember
    bool fixit;                  ///< Признак исправления
    unsigned int errcnt;         ///< Накопительный счетчик ошибок
    ald_string_list &errlist;    ///< Список сообщений об ошибках
    // cppcheck-suppress unusedStructMember
    void* extra;                 ///< Доп. параметр
};

/// Структура события ALD_LDAP_CONSISTENCY
struct ald_ldap_consistency_extra {
    // cppcheck-suppress unusedStructMember
    ald_string_list &modules;   ///< Список необходимых модулей
    ald_string_list &indexes;   ///< Список необходимых индексов
    ald_string_list &uniqueness;///< Список необходимых уникальностей
    // cppcheck-suppress unusedStructMember
    void* extra;                ///< Доп. параметр
};

/// Пара <событие,запись о триггере>
typedef pair<string, ald_action_trigger> ald_trigger_pair;

/// Мультикарта (событие => запись о триггере>
typedef multimap<string, ald_action_trigger> ald_triggers_map;

/// Структура информации о правах файла
struct ald_file_perm {
    string	filename;   ///< Путь к файлу
    string	owner;      ///< Владелец
    string	group;      ///< Группа
    // cppcheck-suppress unusedStructMember
    int		perm;       ///< Права доступа
};

/// Список информации о правах файлов
typedef list<ald_file_perm> ald_file_perm_list;

/// Структура события ALD_BACKUP_FILES
struct ald_backup_extra {
    ald_file_perm_list lst_keys;    ///< Список ключевых файлов
    ald_file_perm_list lst_bases;   ///< Список баз данных
    ald_file_perm_list lst_perms;   ///< Список специальных прав доступа
    // cppcheck-suppress unusedStructMember
    void* extra;                    ///< Доп. параметр
};

/*! \class IALDCore
 * @brief Абстрактный класс IALDCore - основной интерфейс клиента Astra Linux Directory,
 * с помощью которого выполняется взаимодействие со всеми механизмами Astra Linux Directory
 */
class IALDCore
{
public:
    // Properties
    /// Возвращает указатель на провайдер протоколирования
    virtual CALDLogProvider* LogProvider() = 0;

    /// Функции получения и работы с внутренними параметрами ядра ALD
    ///@{
    virtual const string &domain() = 0;
    virtual const string &serverName() = 0;
    virtual const string &hostName() = 0;
    virtual const string &compatibilityVersion() = 0;
    virtual const string &ApplicationName() = 0;
    virtual void SetApplicationName(const string& name) = 0;

    virtual const ald_string_list &HiddenPrincipals() = 0; 		// ALD hidden Principals
    virtual const ald_string_list &ServicesGroups() = 0;   		// ALD groups of services
    virtual const ald_string_list &SupportedFS() = 0;      		// Supported fs types
    virtual const ald_string_list &DefaultLocalGroups() = 0;    // Default local groups
    virtual const ald_string_list &AllowedLocalGroups() = 0;    // Allowed local groups
    virtual ald_string_map &Tags() = 0;                // Tags
    virtual string ALDSubdir() = 0;
    ///@}

    // States and conditions
    /// Возвращает признаки состояния ядра ALD
    ///@{
    virtual bool IfForce() = 0;
    virtual bool IfVerbose() = 0;
    virtual bool Ifrfc2822() = 0;
    virtual bool IfServer() = 0;
    virtual bool IfServerOn() = 0;
    virtual bool IfClientOn() = 0;
    virtual bool IfUseFS(const string &fs) = 0;
    virtual bool IfNotUseFS() = 0;
    virtual bool IfConfigured() = 0;
    virtual bool IfUseDNS() = 0;
    virtual bool isServerHost() = 0;
    ///@}

    // modules (configurators, interfaces, plugins)
    /// Функции работы с модулями ALD
    ///@{
    virtual void ReloadExtensions() = 0;
    virtual void InitPlugins(const string &mode) = 0;
    virtual const ald_module_info &HostModuleInfo() = 0;
    virtual void SetHostModuleInfo(const string &name, const string &desc) = 0;
    virtual void EnumerateModules(int module_type, ald_string_map &mods) = 0;
    virtual bool IfModuleLoaded(const string &name) = 0;
    ///@}

    // configurators
    /// Функции работы с конфигураторами ALD
    ///@{
    virtual void EnumerateConfigurators(ald_string_list &cfgs) = 0;
    virtual void AddConfigurator(CALDConfiguratorPtr cfgr) = 0;
    virtual void RemoveConfigurator(const string &name) = 0;
    virtual void UseConfigurators(const string &mode) = 0;
    virtual void OperationConfig(const string &mode, const string &operation) = 0;
    ///@}

    // interfaces
    /// Функции работы с интерфейсами ALD
    ///@{
    virtual void EnumerateInterfaces(ald_string_list &itfs) = 0;
    virtual void AddInterface(CALDInterfacePtr itf) = 0;
    virtual CALDInterfacePtr GetInterface(const string &name) = 0;
    virtual void RemoveInterface(const string &name) = 0;
    virtual void UseInterface(const string &name, const string &mode) = 0;
    ///@}

    // validators
    /// Функции работы с валидаторами ALD
    ///@{
    virtual void EnumerateValidators(ald_string_list &vlds) = 0;
    virtual void AddValidator(CALDValidatorPtr vld) = 0;
    virtual CALDValidatorPtr GetValidator(const string &name) = 0;
    virtual void RemoveValidator(const string &name) = 0;
    virtual bool ValidateValue(const string &validator, const void *value, const void *tag, void *result, string *err) = 0;
    ///@}

    // triggers
    /// Функции работы с триггерами ALD
    ///@{
    virtual void EnumerateTriggers(const string &action, ald_string_map &trgs) = 0;
    virtual void RegisterTrigger(const string &action, const string &name, ald_action_trigger_func &trigger) = 0;
    virtual void UnregisterTrigger(const string &action, const string &name, ald_action_trigger_func &trigger) = 0;
    virtual bool TriggerAction(const string &action, const string &object, CALDCommand *cmd, void* extra) = 0;
    ///@}

    // Admin, Secrets, Login
    /// Функции работы с аутентификацией и авторизацией администратора
    ///@{
    /// Имя текущего аутентифицированного пользователя (администратора)
    virtual string admin() const = 0;
    /// Текущий способ аутентификации
    virtual ALDAuthType authType() const = 0;
    virtual bool IfKadmin() const = 0;
    virtual bool IfCCache() const = 0;
    virtual void SetCCache(bool ccache) = 0;
    virtual bool GetSecret(const string &user, string &secret, const string &prompt = "") = 0;
    virtual bool NewSecret(const string &user, string &secret, const string &prompt = "") = 0;
    virtual void SetSecret(const string &user, const string &secret) = 0;
    virtual void LoginAdmin(const string &user, ALDAuthType at, bool kadmin = true) = 0;
    /// Удаление аутентификационной информации текущего пользователя
    virtual void ForgetAdmin() = 0;
    ///@}

    // Signals
    /// Функции работы с сигналами
    ///@{
    virtual void UseSigInt() = 0;
    virtual void SetSigIntMode(asmSigIntMode mode) = 0;
    virtual asmSigIntMode GetSigIntMode() = 0;
    virtual bool IfSigInt() = 0;
    virtual bool IfSigPipe() = 0;
    virtual bool IfSigTerm() = 0;
    virtual bool IfSigHup() = 0;
    virtual void ClearSignal(int sig) = 0;
    ///@}

    // Settings
    /// Функции работы с параметрами ALD
    ///@{
    virtual void LoadSettings() = 0;
    virtual string SettingsValue(const string &p) = 0;
    virtual bool SettingsValueExists(const string &p) = 0;
    virtual void SetSettingsValue(const string &p, const string &v) = 0;
    ///@}

    // Commands and options
    /// Функции работы с командами ALD
    ///@{
    virtual CALDCommands &CLCommands() = 0;
    virtual CALDCommandOptions &CommonCLOptions() = 0;
    virtual void OnBeforeCommand() = 0;
    virtual void OnAfterCommand() = 0;
    virtual int ExecALDCommand(const string &command, const string &argument, const ald_string_multimap &mapOptions) = 0;
    virtual int ExecALDCommand(const string &command, const ald_string_list &args, const ald_string_multimap &mapOptions) = 0;
    ///@}

    // Utils
    /*! \brief Сопоставление по регулярному выражению
     * @param[in] pattern Шаблон
     * @param[in] value Значение
     * @param[in] optc Параметр RE
     * @param[in] opte Параметр RE
     * @return true - если подходит
     */
    virtual bool RE_match(const string &pattern, const string &value, const int optc = 0, const int opte = 0) = 0;

    /// Проверка разрешения имен
    virtual void CheckNameResolution() = 0;

    /// Установка режима работы с Kerberos
    virtual void SetKadmLocal(bool kadmlocal) = 0;

    /// Функции вывода информации ядра
    ///@{
    virtual void ShowSettings(ostream &out) = 0;
    virtual void ShowInternals(ostream &out) = 0;
    virtual void ShowStatus(ostream &out) = 0;
    ///@}

    // lists
    /// Модификация внутреннего списка групп сервисов
    virtual bool ServiceGroupsMod(const string &sgroup, bool remove = false) = 0;
    /// Модификация внутреннего списка поддерживаемых ФС
    virtual bool SupportedFSMod(const string &fs, bool remove = false) = 0;

    // Maintenance
    virtual bool MaintenanceMode() = 0;
};

/*! \name Базовые константы */
///@{
#define ALD_USER_NAME_MAX_SIZE		31
#define ALD_GROUP_NAME_MAX_SIZE		ALD_USER_NAME_MAX_SIZE
#define ALD_PWD_MAX_SIZE		    40
#define ALD_DESC_MAX_SIZE		    1000
#define ADM_USER_NAME             "admin/admin"
#define ADM_ALDD_USER_NAME        "aldd"
#define ADM_DEF_USER_NAME         "default/user"
#define ALD_DEF_POLICY_NAME       "default"
///@}

///@{
/*! \brief Типы ФС */
#define ALD_FS_LOCAL              "local"
#define ALD_FS_NFS                "nfs"
#define ALD_FS_CIFS               "cifs"
///@}

/*! \name Константы имен общих параметров */
///@{
#define ALD_PRM_VERSION           "VERSION"
#define ALD_PRM_SERVER            "SERVER"
#define ALD_PRM_DOMAIN            "DOMAIN"
#define ALD_PRM_DOMAIN_DN         "DOMAIN_DN"
#define ALD_PRM_REALM             "REALM"
#define ALD_PRM_NETWORK_FS_TYPE   "NETWORK_FS_TYPE"
#define ALD_PRM_MINIMUM_UID       "MINIMUM_UID"
#define ALD_PRM_MINIMUM_GID       "MINIMUM_GID"
#define ALD_PRM_DEF_LOGIN_SHELL   "DEFAULT_LOGIN_SHELL"
#define ALD_PRM_TICKET_MAX_LIFE   "TICKET_MAX_LIFE"
#define ALD_PRM_TICKET_MAX_R_LIFE "TICKET_MAX_RENEWABLE_LIFE"
#define ALD_PRM_DESC              "DESCRIPTION"
#define ALD_PRM_RPC_PORT          "RPC_PORT"
#define ALD_PRM_USE_RPC           "USE_RPC"
#define ALD_PRM_ALDD_USER         "ALDD_USER"
#define ALD_PRM_USE_DNS           "USE_DNS"
#define ALD_PRM_USER_GROUPS       "USER_GROUPS"
#define ALD_PRM_USE_DOCUMENTS     "USE_DOCUMENTS"
#define ALD_PRM_PREF_SERVERS      "PREFERRED_SERVERS"
#define ALD_PRM_AUTORECONF        "AUTORECONFIGURE"
#define ALD_PRM_VALID_USER_NAMES  "VALID_USER_NAMES"
#define ALD_PRM_VALID_GROUP_NAMES "VALID_GROUP_NAMES"
#define ALD_PRM_ALD_HOSTNAME      "ALD_HOSTNAME"
///@}

/*! \name Константы имен параметров клиента */
///@{
#define ALD_PRM_CLIENT_MOUNT_DIR   	  "CLIENT_MOUNT_DIR"
#define ALD_PRM_CLIENT_ON          	  "CLIENT_ON"
#define ALD_PRM_CLIENT_FS_KRB_MODE 	  "CLIENT_FS_KRB_MODE"
#define ALD_PRM_CLIENT_SMB_VERSION 	  "CLIENT_SMB_VERSION"
#define ALD_PRM_ALLOWED_LOCAL_GROUPS  "ALLOWED_LOCAL_GROUPS"
#define ALD_PRM_CACHE_REFRESH_PERIOD  "CACHE_REFRESH_PERIOD"
#define ALD_PRM_CACHE_ACCESS_USERS    "CACHE_ACCESS_USERS"
#define ALD_PRM_LOCAL_SOCKET_PATH     "LOCAL_SOCKET_PATH"
///@}

/*! \name Константы имен параметров сервера */
///@{
#define ALD_PRM_SERVER_ON               "SERVER_ON"
#define ALD_PRM_SERVER_EXPORT_DIR       "SERVER_EXPORT_DIR"
#define ALD_PRM_SERVER_ARCHIVE_DIR      "SERVER_ARCHIVE_DIR"
#define ALD_PRM_SERVER_FS_KRB_MODES     "SERVER_FS_KRB_MODES"
#define ALD_PRM_SERVER_POLLING_PERIOD   "SERVER_POLLING_PERIOD"
#define ALD_PRM_SERVER_PROPAGATE_PERIOD "SERVER_PROPAGATE_PERIOD"
#define ALD_PRM_UTF8_GECOS              "UTF8_GECOS"
#define ALD_PRM_DEFAULT_LOCAL_GROUPS    "DEFAULT_LOCAL_GROUPS"
#define ALD_PRM_SERVER_ID               "SERVER_ID"
#define ALD_PRM_SERVER_RPC_RESTRICTED   "RPC_RESTRICTED"
#define ALD_PRM_SERVER_KEYTAB           "SERVER_KEYTAB"
#define ALD_PRM_SERVER_KADM_RELOAD      "SERVER_KADM_RELOAD"
///@}

/*! \name Константы имен внутренних сервера */
///@{
#define ALD_PRM_SUB_DIR           "ALD_SUB_DIR"
#define ALD_PRM_CC_PREFIX         "ALD_CC_PREFIX"
#define ALD_PRM_VAR_LIB_DIR       "ALD_VAR_LIB_DIR"
#define ALD_PRM_COMMON_KEYTAB     "ALD_COMMON_KEYTAB"
#define ALD_PRM_CFG_TEMPL_DIR     "ALD_CFG_TEMPL_DIR"
#define ALD_PRM_CFG_DIR           "ALD_CFG_DIR"
#define ALD_PRM_ITFS_DIR          "ALD_ITFS_DIR"
#define ALD_PRM_PLUGINS_DIR       "ALD_PLUGINS_DIR"
#define ALD_PRM_CFG_MODULES_DIR   "ALD_CFG_MODULES_DIR"
#define ALD_PRM_CFG               "ALD_CFG"
#define ALD_PRM_CFG_ROOT_DIR      "ALD_CFG_ROOT_DIR"
#define ALD_PRM_USR_SHARE_DIR     "ALD_USR_SHARE_DIR"
#define ALD_PRM_LDAP_SCHEMA_DIR   "ALD_LDAP_SCHEMA_DIR"
#define ALD_PRM_CACHE_DIR         "ALD_PRM_CACHE_DIR"
#define ALD_PRM_DOMAIN_USERS_GID  "ALD_DOMAIN_USERS_GID"
#define ALD_PRM_DOCS_DIR          "ALD_PRM_DOCS_DIR"
#define ALD_PRM_LDAP_URIS         "ALD_PRM_LDAP_URIS"
#define ALD_PRM_CONN_LOCK_FILE    "ALD_PRM_CONN_LOCK_FILE"

///@}

/*! \name Имена базовых валидаторов */
///@{
#define ALD_VALIDATOR_USER_NAME         "UserName"
#define ALD_VALIDATOR_GROUP_NAME        "GroupName"
#define ALD_VALIDATOR_PATH              "Path"
#define ALD_VALIDATOR_GECOS             "Gecos"
#define ALD_VALIDATOR_POLICY_NAME       "PolicyName"
#define ALD_VALIDATOR_POLICY_LIFETIME   "PolicyLifeTime"
#define ALD_VALIDATOR_TASK_HISTORY      "TaskHistory"
#define ALD_VALIDATOR_SERVICE_NAME      "ServiceName"
#define ALD_VALIDATOR_HIDDEN_PRINCIPALS "HiddenPrincipals"
#define ALD_VALIDATOR_HOST_NAME         "HostName"
#define ALD_VALIDATOR_DOMAIN_NAME       "DomainName"
#define ALD_VALIDATOR_HOST_FLAGS        "HostFlags"
#define ALD_VALIDATOR_SGROUP_NAME       "SvcGroupName"
#define ALD_VALIDATOR_HGROUP_NAME       "HostGroupName"
#define ALD_VALIDATOR_DNS_NAME          "DNSName"
#define ALD_VALIDATOR_DOC_NAME          "DocumentName"
///@}

/*! \name Имена секций помощи */
///@{
#define ALD_CMD_SECTION     "ALD Common API"
///@}

/*! \name Имена встроенных объектов домена */
///@{
#define ALD_DOMAIN_USERS        "Domain Users"
#define ALD_DOMAIN_COMPUTERS    "Domain Computers"
#define ALD_DOMAIN_CONTROLLERS  "Domain Controllers"
#define ALD_DOMAIN_FILESERVERS  "Domain File Servers"
///@}

///@}
} // ALD

#endif //ALD_CORE
