/*! \file ALDCoreUtils.h
 * \brief Объявление вспомогательных типов и функций работы с ядром Astra Linux Directory
 * \ingroup ALDCoreUtils_group
 */

#ifndef ALD_CORE_UTILS
#define ALD_CORE_UTILS

#include <string>
#include "ALDCore.h"
#include "ldap-wrapper.h"
#include "kadm5-wrapper.h"
#include "ald-config.h"
#include "ald-rpc.h"
#include <stdint.h>
using namespace std;

namespace ALD
{
/*! \defgroup ALDCoreUtils_group Типы и функции работы с ядром (ALDCoreUtils.h)
 * \brief Объявление вспомогательных типов и функций работы с ядром Astra Linux Directory
 * @{
 */

// ALD connection interfaces flags
/*! \name Флаги соединения */
///@{
#define ALD_CI_LDAP     0x1     // LDAP interface needed
#define ALD_CI_KADM5    0x2     // KADM5 interface needed
#define ALD_CI_RPC      0x4     // RPC interface needed

#define ALD_CI_GSS      0x100   // used GSS authentication
#define ALD_CI_LAZY     0x200   // dont establish connection immedeately
#define ALD_CI_CACHE    0x400   // try use credential cache

#define ALD_CI_ALL      ALD_CI_LDAP|ALD_CI_KADM5|ALD_CI_RPC|ALD_CI_GSS|ALD_CI_LAZY
#define ALD_CI_LOCAL    ALD_CI_LDAP|ALD_CI_KADM5
#define ALD_CI_CLIENT   ALD_CI_LDAP|ALD_CI_RPC|ALD_CI_GSS|ALD_CI_LAZY
///@}

/*! \name Функции получения интерфейсов */
///@{
CALDLdapConnPtr GetLdapConnection(IALDCore &Core, ALDAuthType at, unsigned int ci_flags);
CALDKadm5ConnPtr GetKadm5Connection(IALDCore &Core, ALDAuthType at, unsigned int ci_flags);
CALDRpcConnPtr GetRpcConnection(IALDCore &Core, ALDAuthType at, unsigned int ci_flags);
CALDConfigPtr GetConfig(IALDCore &Core);
///@}

/*! \class CALDConnection
 * \brief Класс соединения
 */
class CALDConnection
{
protected:
    IALDCore *m_pCore;              ///< Указатель на интерфейс ядра
    ALDAuthType m_AuthType;         ///< Текущий тип аутентификации
    CALDLdapConnPtr m_connLdap;     ///< Указатель на интерфейс LDAP
    CALDKadm5ConnPtr m_connKadm5;   ///< Указатель на интерфейс Kerberos
    CALDRpcConnPtr m_connRpc;       ///< Указатель на интерфейс RPC
    unsigned int m_ci_flags;        ///< Флаги соединения
    string m_strUserId;             ///< Аутентифицированный пользователь
    bool m_bActive;                 ///< Состояние активного соединения
public:
    /// Конструктор
    explicit CALDConnection(IALDCore &Core, ALDAuthType at,
                            unsigned int flags = ALD_CI_ALL);
    /// Возвращает тип аутентификации
    ALDAuthType AuthType() const;
    /// Возвращает указатель на интерфейс ядра
    IALDCore *pCore() const;
    /// Возвращает указатель на интерфейс LDAP
    CALDLdapConnPtr &ldap();
    /// Возвращает указатель на интерфейс Kerberos
    CALDKadm5ConnPtr &kadm5();
    /// Возвращает указатель на интерфейс RPC
    CALDRpcConnPtr &rpc();
    /// Возвращет аутентифицированного пользователя
    const string& userId() const {return m_strUserId;}

    /// Проверка соединения
    bool Check(unsigned int timeout, unsigned int delay);
    /// Возвращает состояние соединения
    bool IsConnected();
    /// Установка соединения по типу аутентификации и флагам
    void Connect(ALDAuthType at, unsigned int ci_flags = 0);
    /// Установка соединения для заданного пользователя по типу аутентификации и флагам
    void Connect(const string &userId, ALDAuthType at, unsigned int ci_flags = 0);
    /// Завершение соединения
    void Disconnect();
    /// Восстановление соединения
    void EnsureConnection();
    /// Возвращет флаги соединения
    unsigned int flags() const;
    /// Возвращает доступность интерфейса LDAP
    bool IsLdap() const;
    /// Возвращает доступность интерфейса Kerberos
    bool IsKadm() const;
    /// Возвращает доступность интерфейса RPC
    bool IsRpc() const;
    virtual ~CALDConnection();
};

/// Разделяемый указатель на соединение
typedef shared_ptr<CALDConnection> CALDConnectionPtr;

/// Регистрирует указанную схему LDAP
void RegisterLdapSchema(IALDCore &Core, const string &schema);

/// Выделение ядра
IALDCore* AllocateCore();

/// Освобождение ядра
void FreeCore(IALDCore* &ICore);

/// Вход администратора
bool LoginAdmin(IALDCore &Core, const string &admin = "");

/// Вход администратора с паролем
bool LoginAdmin(IALDCore &Core, const string &admin, const string &pwd);

/// Выход администратора
void Logout(IALDCore &Core, bool keep_cred = false);

/// Инициализация ядра
void InitCore(IALDCore &Core);

/*! \class CALDUserNameValidator
 * \brief Класс валидатора имени пользователя
 * \details Regular expression in ALD_PRM_VALID_USER_NAMES => tag ignored, result ignored
 */
class CALDUserNameValidator: public CALDValidator
{
    /// Регулярное выражение имени пользователя
//#define ALD_RE_USER_NAME "[a-z][a-z0-9\\_\\-]*" now in ALD_PRM_VALID_USER_NAMES
public:
    /// Конструктор
    CALDUserNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_USER_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDGroupNameValidator
 * \brief Класс валидатора имени группы
 * \details Regular expression in ALD_PRM_VALID_GROUP_NAMES => tag ignored, result ignored
 */
class CALDGroupNameValidator: public CALDValidator
{
    /// Регулярное выражение имени группы
//#define ALD_RE_GROUP_NAME "[A-Za-z][A-Za-z0-9\\_\\-\\ ]*" now in ALD_PRM_VALID_GROUP_NAMES
public:
    /// Конструктор
    CALDGroupNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_GROUP_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDServiceNameValidator
 * \brief Класс валидатора имени сервиса
 * \details "[A-Za-z][A-Za-z0-9\\_\\-]*" => tag ignored, result (ald_string_map*) ("name", "host", "domain")
 */
class CALDServiceNameValidator: public CALDValidator
{
    /// Регулярное выражение имени сервиса
#define ALD_RE_SERVICE_NAME "[A-Za-z][A-Za-z0-9\\_\\-]*"
public:
    /// Конструктор
    CALDServiceNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_SERVICE_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDPathValidator
 * \brief Класс валидатора пути ФС
 * \details "[\\/0-9A-Za-zА-Яа-я\\.\\-\\_]+" => tag ignored, result (string*)
 */
class CALDPathValidator: public CALDValidator
{
    /// Регулярное выражение пути ФС
#define ALD_RE_PATH "[\\/0-9A-Za-zА-Яа-я\\.\\-\\_]+"
public:
    /// Конструктор
    CALDPathValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_PATH, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDGecosValidator
 * \brief Класс валидатора GECOS
 * \details "[0-9A-Za-zА-Яа-рс-я \\-\\.\\+\\(\\)\\_]*" => tag ignored, result ignored
 */
class CALDGecosValidator: public CALDValidator
{
    /// Регулярное выражение RU GECOS
#define ALD_RE_UTF8_GECOS_FIELD  "[^\\:]*"
//#define ALD_RE_UTF8_GECOS_FIELD  "[0-9A-Za-zА-Яа-рс-я \\@\\#\\$\\-\\.\\+\\(\\)\\_]*"
    /// Регулярное выражение GECOS
#define ALD_RE_GECOS_FIELD  "[^А-Яа-рс-я\\:]*"
//#define ALD_RE_GECOS_FIELD  "[0-9A-Za-z \\@\\#\\$\\-\\.\\+\\(\\)\\_]*"
public:
    /// Конструктор
    CALDGecosValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_GECOS, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDPolicyNameValidator
 * \brief Класс валидатора имени политики
 * \details "[0-9A-Za-z\\_\\-]+" => tag ignored, result ignored
 */
class CALDPolicyNameValidator: public CALDValidator
{
    /// Регулярное выражение имени политики
#define ALD_RE_POLICY_NAME "[0-9A-Za-z\\_\\-]+"
public:
    /// Конструктор
    CALDPolicyNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_POLICY_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDPolicyLifeTimeValidator
 * \brief Класс валидатора времени жизни
 * \details tag ignored, result (unsigned int*) (sec)
 */
class CALDPolicyLifeTimeValidator: public CALDValidator
{
    /// Регулярное выражение времени жизни
#define ALD_RE_LIFETIME "\\d+(d|h|m|s)"
public:
    /// Конструктор
    CALDPolicyLifeTimeValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_POLICY_LIFETIME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDTaskHistoryValidator
 * \brief Класс валидатора размера истории заданий
 * \details tag ignored, result (unsigned int*)
 */
class CALDTaskHistoryValidator: public CALDValidator
{
public:
    /// Конструктор
    CALDTaskHistoryValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_TASK_HISTORY, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDHiddenPrincipalsValidator
 * \brief Класс валидатора скрытых принципалов
 * \details tag ignored, result ignored
 */
class CALDHiddenPrincipalsValidator: public CALDValidator
{
public:
    /// Конструктор
    CALDHiddenPrincipalsValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_HIDDEN_PRINCIPALS, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDHostNameValidator
 * \brief Класс валидатора имени компьютера
 * \details "[a-z0-9\\-][\\.a-z0-9\\-]*" => tag ignored, result (string*)
 */
class CALDHostNameValidator: public CALDValidator
{
    /// Регулярное выражение имени компьютера
#define ALD_RE_HOST_NAME "[a-z0-9\\-][\\.a-z0-9\\-]*"
    /// Регулярное выражение имени домена
#define ALD_RE_HOST_DOMAIN_NAME "[a-z0-9\\-]+%s"
public:
    /// Конструктор
    CALDHostNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_HOST_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDDomainNameValidator
 * \brief Класс валидатора имени домена
 * \details "^\\.[a-z0-9\\-][a-z0-9\\-\\.]*" => tag ignored, result (string*)
 */
class CALDDomainNameValidator: public CALDValidator
{
    /// Регулярное выражение имени домена
#define ALD_RE_DOMAIN_NAME "^\\.[a-z0-9\\-][a-z0-9\\-\\.]*"
public:
    /// Конструктор
    CALDDomainNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_DOMAIN_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDDNSNameValidator
 * \brief Класс валидатора DNS имени
 * \details tag ignored, result (string*)
 */
class CALDDNSNameValidator: public CALDValidator
{
public:
    /// Конструктор
    CALDDNSNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_DNS_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDHostFlagsValidator
 * \brief Класс валидатора флагов компьютера
 * \details tag ignored, result ignored
 */
class CALDHostFlagsValidator: public CALDValidator
{
    /// Регулярное выражение флагов компьютера
#define ALD_RE_HOST_FLAGS "\\[P?C?F?D?\\]"
public:
    /// Конструктор
    CALDHostFlagsValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_HOST_FLAGS, Core){}
    //	tag ignored, result ignored
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDSvcGroupNameValidator
 * \brief Класс валидатора имени группы сервисов
 * \details "[A-Za-z][A-Za-z0-9\\_\\-\\ ]*" => tag ignored, result ignored
 */
class CALDSvcGroupNameValidator: public CALDValidator
{
    /// Регулярное выражение имени группы сервисов
#define ALD_RE_SGROUP_NAME "[A-Za-z][A-Za-z0-9\\_\\-\\ ]*"
public:
    /// Конструктор
    CALDSvcGroupNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_SGROUP_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDHostGroupNameValidator
 * \brief Класс валидатора имени группы компьютеров
 * \details "[A-Za-z][A-Za-z0-9\\_\\-\\ ]*" => tag ignored, result ignored
 */
class CALDHostGroupNameValidator: public CALDValidator
{
    /// Регулярное выражение имени группы компьютеров
#define ALD_RE_HGROUP_NAME "[A-Za-z][A-Za-z0-9\\_\\-\\ ]*"
public:
    /// Конструктор
    CALDHostGroupNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_HGROUP_NAME, Core){}
    //	tag ignored, result ignored
    virtual bool Validate(const void *value, const void *tag, void *result);
};


/*! \class CALDLogTemplateValidator
 * \brief Класс валидатора шаблона протоколирования
 * \details tag ignored, result (ald_string_map*) ("name", "suffix", "mode")
 */
class CALDLogTemplateValidator: public CALDValidator
{
    /// Регулярное выражение шаблона протоколирования
#define ALD_VALIDATOR_LOG_TEMPLATE       "LogTemplate"
public:
    /// Конструктор
    CALDLogTemplateValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_LOG_TEMPLATE, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/*! \class CALDDocNameValidator
 * \brief Класс валидатора имени документа
 * \details tag ignored, result ignored
 */
class CALDDocNameValidator: public CALDValidator
{
public:
    /// Конструктор
    CALDDocNameValidator(IALDCore &Core):CALDValidator(ALD_VALIDATOR_DOC_NAME, Core){}
    virtual bool Validate(const void *value, const void *tag, void *result);
};

/// Регистрация базовых валитаторов
void RegisterBaseValidators(IALDCore &Core);

/// Синтез полного имени компьютера
bool UnparseHostName(IALDCore &Core, const string &host, const string &domain, string &fullname, string *perror);
/// Разбор полного имени компьютера
bool ParseHostName(IALDCore &Core, const string &fullname, string &host, string &domain, string *perror);


/*! \brief Обновление ключей принципала в ключевом файле
* @param[in] krb5 Разделяемый указатель на соединение Kerberos
* @param[in] keytab Путь к файлу ключей
* @param[in] name Имя принципала
* @param[in] create Флаг создания сервиса при отсутствии
* @return Успех или отказ
*/
void UpdatePrincipalInKeytab(CALDKadm5ConnPtr krb5, const string &keytab, const string &name, bool create);

/// Получение имени принципала компьютера
string HostPrincipal(const string &name);
/// ПОлучение DN по имени пользователя
string UserToDN(const string &user, const string &base);
/// ПОлучение имени пользователя по DN
string DNToUser(const string &dn);

/*! \name Управление службами */
///@{
enum ALDServiceOperation {asoStop, asoStart, asoRestart, asoInstall, asoRemove, asoReload};
int ManageService(const string &service, ALDServiceOperation op, bool ignore = false);
// int ManageService(const string &service, const string &op, bool ignore = false);
///@}

/*! \name Исполнение внешних команд */
///@{
int ExecCommand(const string &command, bool infinite = false);
int ExecCommandOut(const string &command, ald_string_list &outlist, bool infinite = false);
int ExecCommandInOut(const string &command, ald_string_list &outlist,
                     const ald_string_list &inlist, bool infinite = false);
///@}

/// Проверка кода возврата по списку успешных
void CheckValidExits(const string &command, const int exitcode, ald_int_list &validlist);

/// Перезапуск служб NSCD, NSLCD
void RestartNSCD();

/// Обновление кеша службы NSCD
void InvalidateNSCD();

/*! \name Работа с клиентской точкой монтирования домашних каталогов */
///@{
void CreateClientMountPoint(IALDCore &Core);
void UmountClientMountPoint(IALDCore &Core, const string &mode);
void MountClientMountPoint(IALDCore &Core, const string &mode);
///@}

/// Установка признака выдачи билетов AllowTix всех принципалов
void SetAllowTix(CALDKadm5ConnPtr &krb5, bool AllowTix);

/// Обновление времени жизни всех принципалов
void UpdateLifetimes(IALDCore &Core, CALDKadm5ConnPtr &krb5);

/// Отбрасывание из списка принципалов записей с '/'
void FilterUserPrincipals(IALDCore &Core, ald_string_list &list);

/// Проверка настроенности ALD
void CheckConfigured(IALDCore &Core);

/// Проверка совместимости с версией
void CheckCompatibility(IALDCore &Core, const string &version);

/// Канонизация имени принципала (синтез имени с доменом)
string CannonicalizePrincipalName(IALDCore &Core, const string &name, const string &domain);

/// Синтез принципала сервиса по имени, компьютеру и домену
string SvcPrincipal(IALDCore &Core, const string &svc, const string &host = "", const string &realm = "");

/// Получение ткстового представления времени жизни
string LifetimeToStr(unsigned int ltime);

/// Разрешение использования указанной общей команды
void UseCommonCommand(IALDCore &Core, const string &cmd);

// serialization
/*! \name Функции сериализации объектов Kerberos */
///@{
string princ2str(const CALDKrbPrincipalPtr &princ);
CALDKrbPrincipalPtr str2princ(const string &name, const string &body);

string policy2str(const CALDKrbPolicyPtr &policy);
CALDKrbPolicyPtr str2policy(const string &name, const string &body);
///@}

// recheck ALD server name
/// Обнаружение контроллера указанного домена
bool GetActiveALDServerName(IALDCore& Core, const string& domain, string& dc);

///@}

/// Установка значения именованного счётчика
void SetCounter(string &name, int value);

/// Получение значения именованного счётчика
int GetCounter(string &name);

/// Атомарное изменение значения именованного счётчика
int IncrementCounter(string &name, int increment);

bool ReConfigureSSSD(IALDCore *pCore, CALDConnectionPtr conn);

void UnConfigureSSSD(IALDCore *pCore);

} // ALD

#endif // ALD_CORE_UTILS
