/*! \file ALDException.h
 * \brief Объявление классов исключений и кодов возврата Astra Linux Directory
 * \ingroup ALDExceptions_group
*/

#ifndef ALD_EXCEPTION
#define ALD_EXCEPTION

#include <string>
#include <list>
#include <exception>
#include "cerrno"

namespace ALD
{

using namespace std;
/*! \defgroup ALDExceptions_group Классы исключений и кодов возврата (EALDError)
 * \brief Работа с исключениями Astra Linux Directory
 * @{
 */

/*! \name Макросы генерации исключений */
///@{
#define THROW_ALD_ERROR(err, msg) throw err(msg, "")
#define THROW_ALD_ERROR_DET(err, msg, det) throw err(msg, det)
#define THROW_TRACE_ALD_ERROR(err, msg) throw err(msg, "", __FILE__, __FUNCTION__, __LINE__)
#define THROW_TRACE_ALD_ERROR_DET(err, msg) throw err(msg, det, __FILE__, __FUNCTION__, __LINE__)

#define ALD_ERROR(msg) THROW_ALD_ERROR(EALDError, msg)
#define ALD_ERROR_DET(msg, det) THROW_ALD_ERROR_DET(EALDError, msg, det)

#define ALD_CHECK_PTR_DESC(ptr, desc) {if(!(ptr)) THROW_TRACE_ALD_ERROR(EALDOutOfMemory, desc);}
#define ALD_CHECK_PTR(ptr) ALD_CHECK_PTR_DESC(ptr, "")

#define ALD_INTERNAL_ERROR(msg) THROW_TRACE_ALD_ERROR(EALDInternalError, msg)

#define ALD_CHECK(condition, desc) {if(!(condition)) THROW_ALD_ERROR(EALDCheckError, desc);}

#define ALD_SIGINT throw EALDSigIntError()

#define ALD_SIGTERM throw EALDSigTermError()

#define ALD_PERM_ERROR(msg) THROW_ALD_ERROR(EALDPermError, msg)
///@}

/*! \class EALDError
 * \brief Базовый класс исключения Astra Linux Directory
 */
class EALDError: public exception
{
protected:
    string m_strMsg;        ///< Сообщение
    string m_strDetails;    ///< Дополнительная информация
    string m_strWhat;       ///< Сформированная строка What
    int m_ierrcode;         ///< Код ошибки
    /// Генерация What
    virtual void makewhat();
public:
    /*! \brief Конструктор исключения
     * @param[in] strMsg Сообщение
     * @param[in] strDetails Дополнительная информация
     */
    explicit EALDError(const string &strMsg, const string &strDetails);

    /*! \brief Конструктор внутреннего исключения с информацией о месте возникновения
     * @param[in] strMsg Сообщение
     * @param[in] strDetails Дополнительная информация
     * @param[in] strFile Имя файла
     * @param[in] strFunc Имя функции
     * @param[in] ulLine Номер строки
     */
    explicit EALDError(const string &strMsg, const string &strDetails, const string &strFile, const string &strFunc, unsigned long ulLine);
    /// Возвращает типовое сообщение what
    virtual const char* what() const throw();
    /// Возвращает сообщение
    virtual const char* msg() const throw();
    /// Возвращает дополнительную информацию
    virtual const char* details() const throw();
    /// Возвращает код ошибки
    virtual int errcode() const;
    virtual ~EALDError() throw();
};

/*! \class EALDOutOfMemory
 * \brief Класс исключения ошибки выделения памяти
 */
class EALDOutOfMemory: public EALDError
{
public:
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError */
    explicit EALDOutOfMemory(const string &strMsg, const string &strDetails);
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError(const string &, const string &, const string &, const string &, unsigned long) */
    explicit EALDOutOfMemory(const string &strMsg, const string &strDetails, const string &strFile, const string &strFunc, unsigned long ulLine);
    virtual ~EALDOutOfMemory() throw();
};

/*! \class EALDInternalError
 * \brief Класс исключения по внутренней ошибке
 */
class EALDInternalError: public EALDError
{
public:
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError */
    explicit EALDInternalError(const string &strMsg, const string &strDetails);
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError(const string &, const string &, const string &, const string &, unsigned long) */
    explicit EALDInternalError(const string &strMsg, const string &strDetails, const string &strFile, const string &strFunc, unsigned long ulLine);
    virtual ~EALDInternalError() throw();
};

/*! \class EALDCheckError
 * \brief Класс исключения проверки
 */
class EALDCheckError: public EALDError
{
public:
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError */
    explicit EALDCheckError(const string &strMsg, const string &strDetails);
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError(const string &, const string &, const string &, const string &, unsigned long) */
    explicit EALDCheckError(const string &strMsg, const string &strDetails, const string &strFile, const string &strFunc, unsigned long ulLine);
    virtual ~EALDCheckError() throw();
};

/*! \class EALDSigIntError
 * \brief Класс исключения сигнала прерывания SigInt
 */
class EALDSigIntError: public EALDError
{
public:
    explicit EALDSigIntError();
    virtual ~EALDSigIntError() throw();
};

/*! \class EALDPermError
 * \brief Класс исключения отказа доступа
 */
class EALDPermError: public EALDError
{
public:
    /*! \brief Конструктор исключения аналогичный EALDError::EALDError */
    explicit EALDPermError(const string &strMsg, const string &strDetails);
    virtual ~EALDPermError() throw();
};

// ALD error code for exit status
/*! \name Константы ошибок Astra Linux Directory */
///@{
#define ALD_ERR_BASE            100
#define ALD_ERR_NO_ERR          0
#define ALD_ERR_OUT_OF_MEMORY   ENOMEM
#define ALD_ERR_UNKNOWN         ALD_ERR_BASE+1
#define ALD_ERR_INTERNAL        ALD_ERR_BASE+2
#define ALD_ERR_KERBEROS        ALD_ERR_BASE+3
#define ALD_ERR_LDAP            ALD_ERR_BASE+4
#define ALD_ERR_EXEC            ALD_ERR_BASE+5
#define ALD_ERR_CONFIG          ALD_ERR_BASE+6
#define ALD_ERR_CONDITIONS      ALD_ERR_BASE+7
#define ALD_ERR_IN_FORCE        ALD_ERR_BASE+8
#define ALD_ERR_NOT_FOUND       ALD_ERR_BASE+9
#define ALD_ERR_REFUSE          ALD_ERR_BASE+10
#define ALD_ERR_FILE_IO         ALD_ERR_BASE+11
#define ALD_ERR_USAGE           ALD_ERR_BASE+12
#define ALD_ERR_PERMISSION      ALD_ERR_BASE+13
///@}

///@}
} // ALD

#endif //ALD_EXCEPTION
