/*! \file ALDLogProvider.h
 * \brief Объявление подсистемы регистрации событий Astra Linux Directory
 * \ingroup CALDLogProvider_group
 */

#ifndef TRACE_LOG
#define TRACE_LOG

#include <cstdio>
#include <string>
#include <ctime>
#include <sys/timeb.h>
#include <cstdarg>
#include <climits>

using namespace std;

namespace ALD
{

/*! \defgroup CALDLogProvider_group Механизм журналирования Astra Linux Directory (LogProvider)
 * \brief Механизм журналирования Astra Linux Directory
 * @{
 */

/// Получение механизма журналирования
#define LOG_PROVIDER  CALDLogProvider::GetLogProvider()

/// Инициализация механизма журналирования
#define INIT_LOG_PROVIDER CALDLogProvider::InitLogProvider()

/// Освобождение механизма журналирования
#define FREE_LOG_PROVIDER CALDLogProvider::ReleseLogProvider()

/*! \name Макросы вывода сообщений */
///@{
#define TRACE_FUNC_NAME	__PRETTY_FUNCTION__

#define TRACE_BEG_PREFIX	"{"
#define TRACE_END_PREFIX	"}"
#define TRACE_THREAD_ID pthread_self()

#define TRACE_FB do{if(LOG_PROVIDER->m_LogSensitivity >= llTrace) LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__, TRACE_BEG_PREFIX);}while(0)
#define TRACE_FE do{if(LOG_PROVIDER->m_LogSensitivity >= llTrace) LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__, TRACE_END_PREFIX);}while(0)
#define TRACE(msg) do{if(LOG_PROVIDER->m_LogSensitivity >= llTrace) LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__, msg);}while(0)

#define TRACE_ LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__,
#define _TRACE )

#define TRACE_CFB do{if(LOG_PROVIDER->m_LogSensitivity >= llTrace) LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__, "%p %s", this, TRACE_BEG_PREFIX);}while(0)
#define TRACE_CFE do{if(LOG_PROVIDER->m_LogSensitivity >= llTrace) LOG_PROVIDER->Trace(TRACE_FUNC_NAME, __FILE__, __LINE__, "%p %s", this, TRACE_END_PREFIX);}while(0)

#define LOG_ERR(msg) LOG_PROVIDER->Put(llError, loInsertID, msg)
#define LOG_WARN(msg) LOG_PROVIDER->Put(llWarning, loInsertID, msg)
#define LOG_NOTICE(msg) LOG_PROVIDER->Put(llNotice, loInsertID, msg)
#define LOG_INFO(msg) LOG_PROVIDER->Put(llInfo, loInsertID, msg)

#define LOG_DBG(msg) do{if(LOG_PROVIDER->m_LogSensitivity >= llDebug) LOG_PROVIDER->Put(llDebug, loInsertID, msg);}while(0)

#define LOG_IMERR(msg) LOG_PROVIDER->Put(llError, loInsertID | loImmediately, msg)
#define LOG_IMWARN(msg) LOG_PROVIDER->Put(llWarning, loInsertID | loImmediately, msg)
#define LOG_IMNOTICE(msg) LOG_PROVIDER->Put(llNotice, loInsertID | loImmediately, msg)
#define LOG_IMINFO(msg) LOG_PROVIDER->Put(llInfo, loInsertID | loImmediately, msg)

#define LOG_IMDBG(msg) do{if(LOG_PROVIDER->m_LogSensitivity >= llDebug) LOG_PROVIDER->Put(llDebug, loInsertID | loImmediately, msg);}while(0)

#define LOG_OERR(opt, msg) LOG_PROVIDER->Put(llError, opt, msg)
#define LOG_OWARN(opt, msg) LOG_PROVIDER->Put(llWarning, opt, msg)
#define LOG_ONOTICE(opt, msg) LOG_PROVIDER->Put(llNotice, opt, msg)
#define LOG_OINFO(opt, msg) LOG_PROVIDER->Put(llInfo, opt, msg)

#define LOG_ODBG(msg) do{if(LOG_PROVIDER->m_LogSensitivity >= llDebug) LOG_PROVIDER->Put(llDebug, opt, msg);}while(0)

#define LOG_ERR_ LOG_PROVIDER->Put(llError, loInsertID,
#define LOG_WARN_ LOG_PROVIDER->Put(llWarning, loInsertID,
#define LOG_NOTICE_ LOG_PROVIDER->Put(llNotice, loInsertID,
#define LOG_INFO_ LOG_PROVIDER->Put(llInfo, loInsertID,
#define LOG_DBG_ LOG_PROVIDER->Put(llDebug, loInsertID,
#define _LOG )
///@}

/// Уровень журналирования
enum ALDLogLevel {
    llError,    ///< Ошибки
    llWarning,  ///< Предупреждения
    llNotice,   ///< Уведомления
    llInfo,     ///< Информация
    llDebug,    ///< Отладка
    llTrace     ///< Трассировка
};

/// Цели журналирования
enum ALDLogDest {
    ldStdErr = 1,   ///< Поток ошибок stderr
    ldSysLog = 2,   ///< syslog
    ldCSVLog = 4    ///< Текстовый файл
};

/// Опции журналирования
enum ALDLogOption {
    loEmpty = 0,        ///< Отсутствие опций
    loInsertID = 1,     ///< Добавлять идентификатор процесса
    loToFileOnly = 2,   ///< Вывод только в файл
    loImmediately = 4   ///< Немедленный вывод
};

/*! \class CALDLogProvider
 * \brief Базовый класс механизма журналирования Astra Linux Directory
 */
class CALDLogProvider
{
protected:
    FILE* m_pfDebug;        ///< Объект FILE журнала
    string m_strLogFile;    ///< Путь к файлу журнала
    string m_strLogIdent;   ///< Идентификатор (имя) журнала
    struct timeb m_tmb;     ///< Метка времени
    time_t m_tstart;        ///< Время начала журнала
    size_t m_muteStdErr;    ///< Счетчик глушения StdErr

    /*! @{
     * \brief Внутренние функции вывода сообщений в различные типы журналов
     * @param[in] logLevel Уровень сообщения
     * @param[in] ulLogOption Опции журналирования
     * @param[in] format Строка форматирования
     * @param[in] ap Аргументы сообщения
     */
    void InternalPutSysLog(ALDLogLevel logLevel, unsigned long ulLogOption, const char *format, va_list ap);
    void InternalPutStdErr(ALDLogLevel logLevel, unsigned long ulLogOption, const char *format, va_list ap);
    void InternalPutCSVLog(ALDLogLevel logLevel, unsigned long ulLogOption, const char *format, va_list ap);
    /// @}
public:
    bool m_bDetailed;   ///< Признак повышенной детализации вывода
    bool m_bSigInt;     ///< Признак режима прерывания SigInt
    ALDLogLevel m_LogSensitivity;   ///< Уровень журналирования ALD
    unsigned long m_ulLogDests;     ///< Маска целей журналирования

    /// Возвращает путь к файлу журнала
    const string& strLogFile() const {return m_strLogFile; }
    /// Возвращает идентификатор (имя) журнала
    const string& strLogIdent() const {return m_strLogIdent; }

    /*! \brief Добавление форматированного сообщения в журнал
     * @param[in] logLevel Уровень сообщения
     * @param[in] ulLogOption Опции журналирования
     * @param[in] format Строка форматирования
     * @param[in] ... Аргументы сообщения
     */
    void Put(ALDLogLevel logLevel, unsigned long ulLogOption, const char *format, ...);

    /*! \brief Добавление сообщения в журнал
     * @param[in] logLevel Уровень сообщения
     * @param[in] ulLogOption Опции журналирования
     * @param[in] strMsg Сообщение
     */
    inline void Put(ALDLogLevel logLevel, unsigned long ulLogOption, const string& strMsg)
            {Put(logLevel, ulLogOption, strMsg.c_str());}

    /*! \brief Добавление форматированного отладочного сообщения в журнал
     * @param[in] strFn Имя функции
     * @param[in] strFile Имя файла
     * @param[in] ulLine Номер строки
     * @param[in] format Строка форматирования
     * @param[in] ... Аргументы сообщения
     */
    void Trace(const string& strFn, const string& strFile, unsigned long ulLine, const char *format, ...);

    /*! \brief Добавление  отладочного сообщения в журнал
     * @param[in] strFn Имя функции
     * @param[in] strFile Имя файла
     * @param[in] ulLine Номер строки
     * @param[in] strMsg Сообщение
     */
    inline void Trace(const string& strFn, const string& strFile, unsigned long ulLine, const string& strMsg)
            {Trace(strFn, strFile, ulLine, strMsg.c_str());}

    /// Открытие указанного журнала
    void OpenLog(const string& strLogIdent, const string& strFileName = "");
    /// Закрытие журнала
    void CloseLog();
    /// Связывание журнала с указанным файловым дескритором
    void TieToLogFile(int fd);

    /// Глушение/включение COUT
    void MuteStdErr(bool mute);

    /// Статическая функция получения механизма журналирования
    static CALDLogProvider* GetLogProvider();
    /// Инициализация механизма журналирования
    static void InitLogProvider();
    /// Освобождение механизма журналирования
    static void ReleseLogProvider();

    CALDLogProvider();
    ~CALDLogProvider();
    size_t muteCout() const;
    void setMuteCout(const size_t &muteCout);
};

/*
const size_t MAX_CLOCK_T =  (sizeof(clock_t) == 8)? LLONG_MAX: LONG_MAX;

// profiler
class CALDProfiler
{
protected:
    clock_t m_clkStart;
public:
    CALDProfiler(){m_clkStart = clock();}
    inline void start(){m_clkStart = clock();}
    inline clock_t elapsed() const {clock_t clk = clock() - m_clkStart; if(clk < 0) clk += MAX_CLOCK_T; return clk;}
    virtual ~CALDProfiler(){}
};

#define LOG_PROF_EX(msg, lck, ll) LOG_PROVIDER->Put(ll, loInsertID | loImmediately, "%s:%d", msg, lck)
#define LOG_PROF(msg, lck) LOG_PROF_EX(msg, lck, llDebug)

#define CLOCKS_PER_MSEC (CLOCKS_PER_SEC/1000)

#define LOG_PROF_MS_EX(msg, lck, ll) LOG_PROVIDER->Put(ll, loInsertID | loImmediately, "%s:%f", msg, ((float)lck)/CLOCKS_PER_MSEC)
#define LOG_PROF_MS(msg, lck) LOG_PROF_MS_EX(msg, lck, llDebug)
*/
///@}
} // ALD

#endif // TRACE_LOG
