/*! \file ALDObjects.h
 * \brief Объявление классов объектной модели данных Astra Linux Directory
 * \ingroup ALDObjects_group
 */

#ifndef ALD_OBJECTS
#define ALD_OBJECTS

#include "common.h"
#include "ALDCore.h"
#include "ALDCoreUtils.h"
#include "ALDLogProvider.h"
#include "ldap-wrapper.h"
#include <string>
#include <memory>

using namespace std;

namespace ALD
{

/*! \defgroup ALDObjects_group Объектная модель хранения Astra Linux Directory
 * \brief Объявление классов объектной модели данных Astra Linux Directory
 * @{
 */

/// Неверный идентификатор
#define ALD_INVALID_ID    0

// ALD Object types
/*! \name Типы объектов */
///@{
#define ALD_OT_DUMMY        "ALD_DUMMY"
#define ALD_OT_DOMAIN       "ALD_DOMAIN"
#define ALD_OT_USER         "ALD_DOMAIN_USER"
#define ALD_OT_GROUP        "ALD_DOMAIN_GROUP"
#define ALD_OT_TASK         "ALD_TASK"
#define ALD_OT_EVTASK       "ALD_EVTASK"
#define ALD_OT_POLICY       "ALD_DOMAIN_POLICY"
#define ALD_OT_SERVICE      "ALD_DOMAIN_SERVICE"
#define ALD_OT_SVCGROUP     "ALD_DOMAIN_SERVICE_GROUP"
#define ALD_OT_HOST         "ALD_HOST"
#define ALD_OT_HOSTGROUP    "ALD_HOST_GROUP"
#define ALD_OT_ADMGROUP     "ALD_DOMAIN_ADM_GROUP"
#define ALD_OT_LOG_CFG      "ALD_DOMAIN_LOG_CFG"
#define ALD_OT_TD           "ALD_TRUSTED_DOMAIN"
#define ALD_OT_DOCUMENT     "ALD_DOCUMENT"
///@}

// ALD save marks
/// Метка начала переносимой резерной копии
#define ALD_PBK_HEAD         "<<{ ALD Portable Backup File }>>"

/// Префикс метки начала секции объекта переносимой резерной копии
#define ALD_PBK_PREFIX       "<<{ ALDObject: "

/// Метка конца секции объекта переносимой резерной копии
#define ALD_PBK_SUFFIX       ">>}"

/// Метка начала секции объекта переносимой резерной копии
#define ALD_PBK_SIGN(type)   string(ALD_PBK_PREFIX)+type

// ALD save stages
/*! \name Этапы сохранения резервной копии */
///@{
#define ALD_SS_START        "ALD_SS_START"
#define ALD_SS_POLICIES     "ALD_SS_POLICIES"
#define ALD_SS_HOSTS        "ALD_SS_HOSTS"
#define ALD_SS_HOSTGROUPS   "ALD_SS_HOSTGROUPS"
#define ALD_SS_USERS        "ALD_SS_USERS"
#define ALD_SS_GROUPS       "ALD_SS_GROUPS"
#define ALD_SS_SERVICES     "ALD_SS_SERVICES"
#define ALD_SS_SVCGROUPS    "ALD_SS_SVCGROUPS"
#define ALD_SS_DOCUMENTS    "ALD_SS_DOCUMENTS"
#define ALD_SS_ADMGROUP     "ALD_SS_ADMGROUP"
#define ALD_SS_LOG_CFG      "ALD_SS_LOGCFG"
#define ALD_SS_FINISH       "ALD_SS_FINISH"
///@}

/// Статус загрузки объекта
enum aoLoadStatus{
    aolsLoaded,     ///< Успешно загружен
    aolsContinue,   ///< Продолжение загрузки
    aolsError       ///< Ошибка при загрузке
};

/// Статус загрузки поля объекта
enum aoLoadFieldStatus{
    aolfsLoaded,    ///< Успешно загружено
    aolfsUnexpected,///< Неожиданное поле
    aolfsEnd,       ///< Поля кончились
    aolfsError      ///< Ошибка при загрузке
};

/// Операции над объектом
enum aoOperation{
    aooGet,     ///< Получение
    aooCreate,  ///< Создание
    aooModify,  ///< Модификация
    aooRemove   ///< Удаление
};

/// Тип вызова триггера на события с объектом
enum aoTriggerActionType{
    aotatAction,    ///< Действие
    aotatCheck      ///< Проверка
};

/// Уровень информации объекта
typedef int ald_info_level;

/// Изменение уровня информации не требуется
#define ALD_INFO_UNCHANGED  -1

/// Уровень информации объекта по умолчанию
#define ALD_INFO_DEFAULT  0

/*! \name Вычисление имени события по объекту и суффиксу действия */
///@{
#define ALD_OBJECT_ACTION(object, action) string(object)+":"+action

#define ALD_OBJECT_ADDED        "ADDED"
#define ALD_OBJECT_CK_REMOVAL   "CHECK_REMOVAL"
#define ALD_OBJECT_DO_REMOVING  "DO_REMOVING"
#define ALD_OBJECT_REMOVED      "REMOVED"
#define ALD_OBJECT_MODIFIED     "MODIFIED"
///@}

/*! \class CALDObject
 * \brief Абстрактный базовый класс объекта Astra Linux Directory
 */
class CALDObject
{
protected:
    /// Текущий уровень информации
    ald_info_level m_InfoLevel;
    string m_strName;           ///< Наименование
    string m_strBase;           ///< Базовый путь поиска в LDAP
    bool m_bValid;              ///< Признак валидности
    bool m_bSystem;             ///< Признак системного объекта
    string m_strOType;          ///< Тип объекта ALD
    IALDCore *m_pCore;          ///< Указатель на интерфейс ядра
    CALDConnectionPtr m_Conn;   ///< Разделяемый указатель на соединение
    CALDLdapEntPtr m_Entity;    ///< Разделяемый указатель на LDAP сущность объекта

    // abstract !!! must be overloaded in derived classes for saving specific fields
    /*! \brief Перегружаемый вызов сохранения объекта в резервной копии
     * @param[in] os Поток вывода
     * @param[in] verbose Флаг вывода информации
     */
    virtual void InternalSave(ostream &os, bool verbose = false) = 0;

    /*! \brief Перегружаемый вызов загрузки объекта из резервной копии
     * @param[in] is Поток ввода
     * @param[out] nline Номер текущей строки
     * @param[out] line Текущая строка
     * @param[in] verbose Флаг вывода информации
     * @return Статус загрузки поля объекта
     */
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line,
                                      bool verbose = false);

    /*! \brief Статический вызов загрузки поля
     * @param[in] is Поток ввода
     * @param[out] nline Номер текущей строки
     * @param[out] line Текущая строка
     * @param[in] field Имя поля
     * @param[out] value Значение поля
     * @param[in] fuzzy Флаг отложенной обработки ошибок
     * @return Статус загрузки поля объекта
     */
    static aoLoadFieldStatus LoadField(istream &is, size_t &nline, string &line,
                                const string &field, string &value, bool fuzzy = false);

    /*! \brief Статический вызов загрузки множества одноименных полей
     * @param[in] is Поток ввода
     * @param[out] nline Номер текущей строки
     * @param[out] line Текущая строка
     * @param[in] field Имя поля
     * @param[out] values Список значений поля
     * @return Статус загрузки поля объекта
     */
    static aoLoadFieldStatus LoadFields(istream &is, size_t &nline, string &line,
                                 const string &field, ald_string_list &values);

/// Макрос загрузки поля
#define LOAD_FIELD(is, nline, line, field, value) {if(LoadField(is, nline, line, field, value) != aolfsLoaded) return aolsError;}

/// Макрос загрузки множества одноименных полей
#define LOAD_FIELDS(is, nline, line, field, values) {if(LoadFields(is, nline, line, field, values) == aolfsError) return aolsError;}

    // abstract !!! must be overloaded in derived classes, typically fill m_Entity,
    // m_bValid must be set here if m_Entity is not used
    /*! \brief Перегружаемый внутренний вызов получения информации об объекте
     * \details Должен быть перегружен в потомках, как правило заполняет сущность m_Entity, в противном случае
     *  должен быть корректно установлен флаг валидности m_bValid
     * @param[in] object Имя объекта
     * @param[in] level Требуемый уровень информации
     * @param[in] quiet Флаг тихого режима (без исключений)
     */
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false) = 0;

    /*! \brief Проверка после получения информации
     * \details Проверка корректности полученной информации, заполнение вычисляемых полей, установка признака валидности m_bValid
     * @param[in] quiet Флаг тихого режима (без исключений)
     */
    virtual bool ProcessEntity(bool quiet = false);

    /// Обеспечение указанного уровня информации
    virtual void EnsureInfoLevel(ald_info_level level);

    /// Создание внутренней сущности объекта (m_Entity)
    virtual void EnsureEntity();

    /*! \brief Перегружаемый внутренний вызов удаления объекта
     * @param[in] force Флаг принудительного действия
     * @param[in] extra Указатель на произвольную дополнительную информацию
     */
    virtual void InternalRemove(bool force, void *extra = NULL);

    /*! \brief Перегружаемый внутренний вызов создания объекта
     * @param[in] extra Указатель на произвольную дополнительную информацию
     */
    virtual void InternalCreate(void *extra = NULL);

    /*! \brief Перегружаемый внутренний вызов модификации объекта
     * @param[in] setmask Маска модифицируемых полей
     * @param[in] attrs Словарь модифицируемых полей
     * @param[in] extra Указатель на произвольную дополнительную информацию
     */
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap &attrs, void *extra = NULL);

    // abstract !!! must be overloaded in derived classes, typically MSG_*_OBJECT is used
    /*! \brief Перегружаемый внутренний вызов нотификации о действиях с объектом
     * \details Должен быть перегружен, как правило используется MSG_*_OBJECT, например:
     * \code
     *  string msg;
     *  switch(op)
     *  {
     *   case aooGet: break;
     *   case aooCreate: msg = MSG_CREATE_OBJECT(OBJ_USER, arg);
     *       break;
     *   case aooModify: msg = MSG_UPDATE_OBJECT(OBJ_USER, arg);
     *       break;
     *   case aooRemove: msg = MSG_REMOVE_OBJECT(OBJ_USER, arg);
     *       break;
     *  }
     *
     *  if(!detail.empty())
     *       msg += ": "+detail;
     *
     *  if(op != aooGet)
     *       LOG_NOTICE(msg+"...");
     * \endcode
     * @param[in] op Действие
     * @param[in] arg Сообщение
     * @param[in] detail Дополнительная информация
     */
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail) = 0;

    /*! \brief Перегружаемый внутренний вызов начала действия с объектом
     * @param[in] object Название класса объекта
     * @param[in] op Действие
     * @param[in] arg Сообщение
     * @param[in] detail Дополнительная информация
     * @param[in] force Флаг принудительного действия
     */
    virtual bool StartOperation(const string &object, aoOperation op, const string &arg,
                                const string &detail, bool force = false);

    /*! \brief Перегружаемый внутренний вызов завершения действия с объектом
     * @param[in] object Название класса объекта
     * @param[in] op Действие
     * @param[in] process Необходимость постобработки сущности m_Entity
     * @param[in] quiet Флаг тихого режима
     */
    virtual void CommitOperation(const string &object, aoOperation op, bool process = false, bool quiet = false);

    /*! \brief Генерация события об объекте
     * @param[in] action Наименование события
     * @param[in] atype Тип вызова триггера на события с объектом
     * @param[in] quiet Флаг тихого режима
     */
    bool TriggerAction(const string &action, aoTriggerActionType atype, bool quiet = false);
public:
    /*! \brief Конструктор объекта
     * @param[in] type Тип объекта ALD
     * @param[in] Conn Раздеряемый указатель на соединение
     */
    explicit CALDObject(const string &type, CALDConnectionPtr Conn);

    /// Возвращает раздеряемый указатель на соединение
    CALDConnectionPtr conn()  const {return m_Conn;}
    /// Возвращает раздеряемый указатель на LDAP сущность
    CALDLdapEntPtr entity() const {return m_Entity;}
    /// Возвращает текущий уровень информации объекта
    ald_info_level infolevel() const {return m_InfoLevel;}
    /// Возвращает имя объекта
    string const &name() const;
    /// Возвращает базовый путь поиска в LDAP
    string const &base() const;
    /// Возвращает тип объекта ALD
    const string &otype() const {return m_strOType;}
    /// Возвращает признак валидности объекта
    bool IsValid() const {return m_bValid;}
    /// Возвращает признак системного объекта
    virtual bool IsSystemObject() {return false;}

    /// Перегружаемый метод очистки объекта
    virtual void Clear();

    // must be overloaded if some restriction exists
    /*! \brief Проверка удаления
     * \details Должено быть перегружено, если существуют ограничения
     * @param[in] quiet Флаг тихого режима
     */
    virtual bool CheckRemoval(bool quiet = false);

    /*! \brief Повторное полчение информации об объекте
     * @param[in] level Требуемый уровень информации
     * @param[in] quiet Флаг тихого режима
     */
    virtual bool Refresh(ald_info_level level = ALD_INFO_UNCHANGED, bool quiet = false);

    /*! \brief Сохранение в потоке
     * @param[in] os Поток вывода
     * @param[in] verbose Тип вызова триггера на события с объектом
     */
    virtual void Save(ostream &os, bool verbose = false);

    /*! \brief Загрузка объекта из потока
     * @param[in] is Поток ввода
     * @param[out] nline Номер текущей строки
     * @param[in] verbose Флаг вывода информации
     */
    virtual void Load(istream &is, size_t &nline, bool verbose = false);

    /*! \brief Проверка целостности объекта
     * @param[out] msg Диагностическое сообщение
     */
    virtual bool CheckConsistency(string &msg);

    /*! \brief Проверка поля объекта
     * @param[in] property Идентификатор поля
     * @param[in] value Указатель на проверяемое значение
     * @param[out] msg Указатель на сообщение об ошибке
     * @param[in] quiet Флаг тихого режима
     */
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);

    // abstract !!! must be overloaded in derived classes, common structure:
    /*! \brief Перегружаемый метод удаления
     * \details Должен быть перегружен, общая структура вида:
     * \code
     * CheckRemoval();
     *
     * ALD_OBJECT_START_OPERATION_FORCE(aooRemove, m_strName, force);
     *
     * InternalRemove(force);
     *
     * ALD_OBJECT_COMMIT_OPERATION(aooRemove);  // dummy macros!!!
     * \endcode
     * @param[in] force Флаг принудительного действия
     */
    virtual void Remove(bool force = false) = 0;

    // abstract !!! must be overloaded in derived classes, common structure
    /*! \brief Перегружаемый метод получения информации об объекте
     * \details Должен быть перегружен, общая структура вида:
     * \code
     * ALD_OBJECT_START_OPERATION(aooGet, group);
     *
     * InternalGet(group, level, quiet);
     *
     * ALD_OBJECT_COMMIT_OPERATION_PEQ(aooGet);
     *
     * return m_bValid;
     * \endcode
     * @param[in] oname Имя объекта
     * @param[in] level Требуемый уровень информации
     * @param[in] quiet Флаг тихого режима
     * @return Результат выполнения
     */
    virtual bool Get(const string &oname, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false) = 0;

    /// Деструктор объекта
    virtual ~CALDObject();
};

/// Разделяемый указатель на объект CALDObject
typedef shared_ptr<CALDObject> CALDObjPtr;

/*! \name Базовые элементы маски модифицируемых полей  */
///@{
#define ALD_SET_NAME        0x00001
#define ALD_SET_GID         0x00002
#define ALD_SET_DESC        0x00004
#define ALD_SET_MEMBERS     0x00008
#define ALD_SET_UID         0x00010
#define ALD_SET_HOME        0x00020
#define ALD_SET_SHELL       0x00040
#define ALD_SET_FULL_NAME   0x00080
#define ALD_SET_GECOS       0x00100
#define ALD_SET_POLICY      0x00200
#define ALD_SET_PWCHANGE    0x00400
#define ALD_SET_HOME_TYPE   0x00800
#define ALD_SET_CAPS        0x01000
#define ALD_SET_STATE       0x02000
#define ALD_SET_HOME_SRV    0x04000
#define ALD_SET_LGROUPS     0x08000
#define ALD_SET_FLAGS       0x10000
#define ALD_CLR_FAILCNT     0x20000
///@}

/// Вектор ALD привилегий
typedef unsigned int ald_user_caps;

/*! \name Значения разрядов маски ALD привилегий  */
///@{
#define ALD_CAP_ADMIN           0x80000000
#define ALD_CAP_LOCKED          0x40000000
#define ALD_CAP_USER            0x0
#define ALD_CAP_HOST_ADDER      0x1
#define ALD_CAP_ALL_HOST_ADDER  0x2
#define ALD_CAP_ADM_USER        0x4
#define ALD_CAP_LOGIN_MASK      (ALD_CAP_ADMIN | ALD_CAP_HOST_ADDER | ALD_CAP_ALL_HOST_ADDER | ALD_CAP_ADM_USER)
#define ALD_CAP_FULL_MASK       (ALD_CAP_USER | ALD_CAP_ADMIN | ALD_CAP_LOCKED | \
    ALD_CAP_HOST_ADDER | ALD_CAP_ALL_HOST_ADDER | ALD_CAP_ADM_USER)
///@}

/*! \name Уровни информации пользователя  */
///@{
/// name
#define ALD_USER_INFO_0   0
/// uid, gid, desc, fullname, cap, home, shell, gecos, hometype, policy, home server, local groups, mic
#define ALD_USER_INFO_1   1
/// linuxCaps
#define ALD_USER_INFO_2   2
/// host, host groups
#define ALD_USER_INFO_3   3
/// all without Kerberos
#define ALD_USER_INFO_4   4
///@}

/*! \name Идентификаторы полей пользователя */
///@{
#define ALD_USER_PROP_NAME        0  // ALD_SET_NAME
#define ALD_USER_PROP_UID         1  // ALD_SET_UID
#define ALD_USER_PROP_GID         2  // ALD_SET_GID
#define ALD_USER_PROP_CAPS        3  // ALD_SET_CAPS
#define ALD_USER_PROP_HOME        4  // ALD_SET_HOME
#define ALD_USER_PROP_HOMETYPE    5  // ALD_SET_HOME_TYPE
#define ALD_USER_PROP_GECOS       6  // ALD_SET_GECOS
#define ALD_USER_PROP_SHELL       7  // ALD_SET_SHELL
#define ALD_USER_PROP_DESC        8  // ALD_SET_DESC
#define ALD_USER_PROP_FULLNAME    9  // ALD_SET_FULL_NAME
#define ALD_USER_PROP_POLICY     10  // ALD_SET_POLICY
#define ALD_USER_PROP_HOMESRV    11  // ALD_SET_HOME_SRV
#define ALD_USER_PROP_LGROUPS    12  // ALD_SET_LGROUPS
///@}

/*! \class CALDUser
 * \brief Класс ALD - пользователь
 */
class CALDUser: public CALDObject
{
protected:    
    CALDKrbPrincipalPtr m_krbPrinc; ///< Разделяемый указатель на принципал Kerberos
    string m_strUID;                ///< Строковое представление UID пользователя
    uid_t m_UID;                    ///< UID пользователя
    string m_strGID;                ///< Строковое представление GID пользователя
    gid_t m_GID;                    ///< GID пользователя
    ald_user_caps m_Caps;           ///< Вектор ALD привилегий

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void EnsureEntity();
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalRemove(bool force, void *extra = NULL);
    virtual void InternalCreate(void *extra = NULL);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap& attrs, void *extra);
public:
    /*! \brief Конструктор объекта
     * @param[in] Conn Раздеряемый указатель на соединение
     */
    explicit CALDUser(CALDConnectionPtr Conn);
    /// Возвращает строковое представление UID пользователя
    string const &strUID();
    /// Возвращает UID пользователя
    uid_t uid();
    /// Возвращает строковое представление GID пользователя
    string const &strGID();
    /// Возвращает GID пользователя
    gid_t gid();
    /// Вектор ALD привилегий
    ald_user_caps caps();
    /// Вектор Linux привилегий
    ald_linux_caps linuxCaps();

    /*! \name Возвращает ALD привилегии */
    ///@{
    bool IsAdm();
    bool IsHostAdder();
    bool IsAllHostAdder();
    bool IsAdmUser();
    bool IsLocked();
    bool IsFailLocked();
    bool CanLogin();
    ///@}

    /*! \brief Возвращает список разрешенных компьютеров
     * @param[out] hostlist Список компьютеров
     */
    void AllowedHosts(ald_string_list &hostlist);

    /*! \brief Возвращает список разрешенных групп компьютеров
     * @param[out] hgrouplist Список групп компьютеров
     */
    void AllowedHostGroups(ald_string_list &hgrouplist);

    /// Возвращает домашний каталог
    string home();
    /// Возвращаеттип домашнего каталога
    string hometype();
    /// Возвращает сервер домашнего каталога
    string homeserver();
    /// Возвращает GECOS
    string gecos();
    /// Возвращает оболочку
    string shell();

    /// Возвращает описание
    string desc();
    /// Возвращает имя политики
    string policy();
    /// Возвращает имя первичной группы
    string pgroup();
    /// Возвращает список вторичных групп
    void sgroups(ald_string_list &sgrouplist);
    /// Возвращает список локальных групп
    void lgroups(ald_string_list &lgrouplist);
    /// Возвращает полное имя
    string fullname();
    /// Возвращает принципал Kerberos
    CALDKrbPrincipalPtr princ();
    /// Возвращает мандатный уровень целостности
    unsigned int miclevel();
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname, const string &newpwd, const string &newhometype,
                        const string &newhome, const string &newhomesrv, const string &newshell,
                        const string &newgecos, const string &newdesc, const string &newfull_name,
                        const string &newpolicy, const string &newlgroups, gid_t newgid,
                        uid_t newuid, bool needpwchange = false);
    virtual bool Get(const string &user, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &newhometype, const string &newhome,
                        const string &newhomesrv, const string &newshell, const string &newgecos,
                        const string &newdesc, const string &newfull_name, const string &newpolicy,
                        const string &newlgroups, gid_t newgid, uid_t newuid, bool needpwchange);
    /// Функция смены пароля
    virtual void ChangePwd(const string &pwd);
    virtual void Remove(bool force = false);
    /// Установка ALD привилегий
    void SetCaps(ald_user_caps newcaps);
    /// Установка Linux привилегий
    void SetLinuxCaps(ald_linux_caps newcaps);
    /// Установка разрешенных компьютеров и групп
    void SetAllowedHosts(const ald_string_list &hosts, const ald_string_list &hgroups);
    /// Установка мандатного уровня целостности
    void SetMICLevel(unsigned int level);
    /// Установка фотографии
    void SetPhoto(const char* buff, unsigned long len);
    /// Получение фотографии, результат должен быть освобожден (free)
    void* GetPhoto(unsigned long &len); // result must be freed
    virtual bool CheckConsistency(string &msg);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual ~CALDUser();
};

/// Разделяемый указатель на объект CALDUser
typedef shared_ptr<CALDUser> CALDUserPtr;

/*! \name Уровни информации группы */
///@{
/// name
#define ALD_GROUP_INFO_0   0
/// all
#define ALD_GROUP_INFO_1   1
///@}

/*! \name Идентификаторы полей группы */
///@{
#define ALD_GROUP_PROP_NAME        0  // ALD_SET_NAME
#define ALD_GROUP_PROP_GID         1  // ALD_SET_GID
#define ALD_GROUP_PROP_MEMBER      2  // ALD_SET_MEMBERS
#define ALD_GROUP_PROP_DESC        3  // ALD_SET_DESC
///@}

/*! \class CALDGroup
 * \brief Класс ALD - группа
 */
class CALDGroup: public CALDObject
{
protected:
    string m_strGID;        ///< Тестовое представление GID
    gid_t m_GID;            ///< GID

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap& attrs, void *extra);
public:
    /// Конструктор объекта
    explicit CALDGroup(CALDConnectionPtr Conn);

    /// Возвращает GID
    gid_t gid();
    /// Возвращает nестовое представление GID
    string strGID();

    /// Возвращает описание
    string desc();
    /// Возвращает список членов
    void members(ald_string_list &memlist);
    /// Возвращает список первичных членов
    void pmembers(ald_string_list &pmemlist);
    /// Возвращает признак пустоты группы
    bool empty();
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname, const string &newdesc, const ald_string_list &newusers, gid_t newgid);
    virtual bool Get(const string &group, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &newname, gid_t newgid,
                        const string &newdesc, const ald_string_list &newusers);
    virtual bool CheckRemoval(bool quiet = false);
    virtual void Remove(bool force = false);
    /// Добавление члена группы
    void AddMember(const string &user);
    /// Удаление члена группы
    void RmMember(const string &user);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool IsSystemObject();
    virtual ~CALDGroup();
};

/// Разделяемый указатель на объект CALDGroup
typedef shared_ptr<CALDGroup> CALDGroupPtr;

/// Статус/код ошибки задания
typedef int ald_task_status;

/*! \name Значение статуса задания ALD */
///@{
#define ALD_TASK_STATUS_PENDING   0
#define ALD_TASK_STATUS_OK        1
#define ALD_TASK_STATUS_ERROR     -1
///@}

/*! \name Значения фильтра статуса задания ALD */
///@{
#define ALD_TASK_STATUS_FILTER_PENDING  0x01
#define ALD_TASK_STATUS_FILTER_OK       0x02
#define ALD_TASK_STATUS_FILTER_ERROR    0x04
#define ALD_TASK_STATUS_FILTER_ALL      0x07
///@}

/*! \name Уровни информации задания */
///@{
/// ID, Type, Status, Error
#define ALD_TASK_INFO_1   1
/// Timestamp, Data
#define ALD_TASK_INFO_2   2
///@}

/*! \class CALDTask
 * \brief Класс ALD - задание
 */
class CALDTask: public CALDObject
{
protected:
    string m_strType;           ///< Тип задания
    ald_task_status m_Status;   ///< Статус задания

    virtual void InternalSave(ostream &os, bool verbose = false){/* not saved ? */}
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual bool ProcessEntity(bool quiet = false);
public:
    /// Конструктор объекта
    explicit CALDTask(CALDConnectionPtr Conn);
    /// Возвращает идентификатор
    string const &id() const;
    /// Возвращает тип
    string const &type() const;
    /// Возвращает метку времени
    string timestamp();
    /// Возвращает ошибку
    string err() const;
    /// Возвращает данные
    string data();
    /// Возвращает статус
    ald_task_status status() const;
    /// Возвращает описание статуса
    string strStatus() const;
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(unsigned int ID, const string &TaskType, const string &TaskData);
    virtual void Clear();
    virtual bool Get(const string &ID, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Remove(bool force = false);
    /// Установка статуса успешности
    virtual void SetOkState();
    /// Установка статуса ошибки
    virtual void SetErrorState(const string &e);
    virtual ~CALDTask();
};

/// Разделяемый указатель на объект CALDTask
typedef shared_ptr<CALDTask> CALDTaskPtr;

/*! \class CALDEvTask
 * \brief Класс ALD - событие
 */
class CALDEvTask: public CALDTask
{
protected:
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
public:
    /// Конструктор объекта
    explicit CALDEvTask(CALDConnectionPtr Conn);
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(unsigned int ID, const string &TaskType, const string &TaskData);
    virtual bool Get(const string &ID, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual void Remove(bool force = false);
    virtual void SetOkState();
    virtual void SetErrorState(const string &e);
    virtual ~CALDEvTask();
};

/// Разделяемый указатель на объект CALDEvTask
typedef shared_ptr<CALDEvTask> CALDEvTaskPtr;

// Policy properties ALD_KRB_* from krb5-wrapper.h
// setmask ALD_KRB_* from krb5-wrapper.h

/*! \class CALDPolicy
 * \brief Класс ALD - политика
 */
class CALDPolicy: public CALDObject
{
protected:
    unsigned int m_MinLife;         ///< Минимальное время жизни
    unsigned int m_MaxLife;         ///< Максимальное время жизни
    unsigned int m_MinLength;       ///< Минимальная длина
    unsigned int m_MinClasses;      ///< Минимальное число классов
    unsigned int m_HistoryNum;      ///< Длина истории паролей
    CALDKrbPolicyPtr m_krbPolicy;   ///< Указатель на политику Kerberos

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void EnsureEntity();
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalRemove(bool force, void *extra = NULL);
    virtual void InternalCreate(void *extra = NULL);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap &attrs, void *extra = NULL);
public:
    /// Возвращает политику Kerberos
    CALDKrbPolicyPtr krbpolicy() const;
    /// Возвращает минимальное время жизни
    unsigned int min_life() const;
    /// Возвращает максимальное время жизни
    unsigned int max_life() const;
    /// Возвращает минимальную длину
    unsigned int min_length() const;
    /// Возвращает минимальное число классов
    unsigned int min_classes() const;
    /// Возвращает длину истории паролей
    unsigned int history_num() const;
    /// Возвращает максимально число неуспешных попыток
    unsigned int max_fail() const;
    /// Возвращает интервал между попытками
    unsigned int failcnt_interval() const;
    /// Возвращает продолжительность блокировки
    unsigned int lockout_duration() const;
    /// Возвращает список пользователей с такой политикой
    void users(ald_string_list &userlist);
    /// Конструктор объекта
    explicit CALDPolicy(CALDConnectionPtr Conn);
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname, unsigned int newminlife,
                        unsigned int newmaxlife, unsigned int newminlength,
                        unsigned int newminclasses, unsigned int newhistorynum,
                        unsigned int newmaxfail, unsigned int newfailcntinterval,
                        unsigned int newlockoutduration);
    virtual bool Get(const string &policy, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, unsigned int newminlife,
                        unsigned int newmaxlife, unsigned int newminlength,
                        unsigned int newminclasses, unsigned int newhistorynum,
                        unsigned int newmaxfail, unsigned int newfailcntinterval,
                        unsigned int newlockoutduration);
    virtual bool CheckRemoval(bool quiet = false);
    virtual void Remove(bool force = false);
    virtual bool CheckConsistency(string &msg);
    virtual void Recreate(const string &polname);
    virtual bool IsSystemObject();

    virtual ~CALDPolicy();
};

/// Разделяемый указатель на объект CALDPolicy
typedef shared_ptr<CALDPolicy> CALDPolicyPtr;

/*! \class CALDService
 * \brief Класс ALD - сервис
 */
class CALDService: public CALDObject
{
protected:
    CALDKrbPrincipalPtr m_krbPrinc; ///< Принципал Kerberos

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalCreate(void *extra = NULL);
    virtual void EnsureEntity();
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalRemove(bool force, void *extra = NULL);
public:
    /// Конструктор объекта
    explicit CALDService(CALDConnectionPtr Conn);
    /// Возвращает приципал Kerberos
    CALDKrbPrincipalPtr princ() const {return m_krbPrinc;}
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname);
    virtual bool Get(const string &service, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Возвращает список групп сервисов
    void sgroups(ald_string_list &svcgrouplist);
    virtual void Remove(bool force = false);
    virtual ~CALDService();
};

/// Разделяемый указатель на объект CALDService
typedef shared_ptr<CALDService> CALDServicePtr;

/*! \name Идентификаторы полей группы сервисов */
///@{
#define ALD_SGROUP_PROP_NAME        0
#define ALD_SGROUP_PROP_MEMBER      1
///@}

/*! \class CALDSvcGroup
 * \brief Класс ALD - группа сервисов
 */
class CALDSvcGroup: public CALDObject
{
protected:
    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual bool ProcessEntity(bool quiet = false);
public:
    /// Конструктор объекта
    explicit CALDSvcGroup(CALDConnectionPtr Conn);
    /// Возвращает список членов группы
    void members(ald_string_list &memlist);
    /// Возвращает список сервисов группы
    void services(ald_string_list &svclist);
    virtual bool Get(const string &sgroup, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Добавление сервиса в группу
    void AddService(const string &service);
    /// Удаление сервиса из группы
    void RmService(const string &service);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    void Remove(bool force);
    virtual ~CALDSvcGroup();
};

/// Разделяемый указатель на объект CALDSvcGroup
typedef shared_ptr<CALDSvcGroup> CALDSvcGroupPtr;

/*! \name Уровни информации группы компьютеров */
///@{
/// name
#define ALD_HGROUP_INFO_0   0
/// all
#define ALD_HGROUP_INFO_1   1
///@}

/*! \name Идентификаторы полей группы компьютеров */
///@{
#define ALD_HGROUP_PROP_NAME        0
#define ALD_HGROUP_PROP_HOST        1
#define ALD_HGROUP_PROP_DESC        2
///@}

/*! \class CALDHostGroup
 * \brief Класс ALD - группа компьютеров
 */
class CALDHostGroup: public CALDObject
{
protected:
    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap& attrs, void *extra);
    virtual bool ProcessEntity(bool quiet = false);
public:
    /// Конструктор объекта
    explicit CALDHostGroup(CALDConnectionPtr Conn);

    /// Возвращает описание
    string desc();
    /// Возвращает список членов группы
    void members(ald_string_list &memlist);
    /// Возвращает список компьютеров группы
    void hosts(ald_string_list &hlist);
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname, const string &newdesc);
    virtual bool Get(const string &hgroup, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &newname,
                         const string &newdesc, const ald_string_list &newhosts);
    /// Добавление компьютера в группу
    void AddHost(const string &host);
    /// Удаление компьютера из группы
    void RmHost(const string &host);
    virtual bool IsSystemObject();
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual ~CALDHostGroup();
};

/// Разделяемый указатель на объект CALDHostGroup
typedef shared_ptr<CALDHostGroup> CALDHostGroupPtr;

/*! \name Уровни информации группы компьютеров */
///@{
/// name, desc, flags, serverID
#define ALD_HOST_INFO_0   0
/// krb
#define ALD_HOST_INFO_1   1
///@}

/*! \name Идентификаторы полей компьютера */
///@{
#define ALD_HOST_PROP_NAME        0
#define ALD_HOST_PROP_DESC        1
#define ALD_HOST_PROP_FLAGS       2
#define ALD_HOST_PROP_ID          3
///@}

/*! \name Символы флагов компьютера */
///@{
#define ALD_HOST_DC         'C'
#define ALD_HOST_PRIMARY    'P'
#define ALD_HOST_FS         'F'
#define ALD_HOST_DISABLED   'D'

#define ALD_HOST_EMPTY      "[]"
#define ALD_HOST_PDC        "[PCF]"
#define ALD_HOST_BDC        "[CF]"
#define ALD_OLD_SRV         "[PCF]"
///@}

/*! \class CALDHost
 * \brief Класс ALD - компьютер
 */
class CALDHost: public CALDObject
{
protected:
    string m_strFlags;              ///< Строка флагов компьютера
    unsigned int m_serverID;        ///< Идентификатор сервера
    CALDKrbPrincipalPtr m_krbPrinc; ///< Приципал Kerberos

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void EnsureEntity();
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalCreate(void *extra = NULL);
    virtual void InternalRemove(bool force, void *extra = NULL);
public:
    /// Возвращает принципал Kerberos
    CALDKrbPrincipalPtr princ();

    /// Конструктор объекта
    explicit CALDHost(CALDConnectionPtr Conn);

    /// Возвращает описание
    string desc();
    /// Возвращает строку флагов компьютера
    string const flags() const;
    /// Возвращает идентификатор сервера
    unsigned int serverID();
    /*! \name Возвращает флаги компьютера */
    ///@{
    bool IsPDC();
    bool IsBDC();
    bool IsFS();
    bool IsDisabled();
    ///@}
    /// Возвращает список групп компьютеров
    void hgroups(ald_string_list &hgrouplist);
    /// Возвращает список сервисов компьютера
    void hservices(ald_string_list &svclist);
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newhost, const string &newdesc);
    virtual bool Get(const string &host, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта
    virtual void Update(const string &hostdesc);
    /// Установка новых флагов и идентификатора сервера
    virtual void SetFlags(const string &newflags, unsigned int newID);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    virtual ~CALDHost();
};

/*! \brief Модификация строки флагов компьютера
 * @param[in] flags Исходная строка флагов
 * @param[in] flag Символ флага
 * @param[in] op Операция (true - add, false - rm)
 * @return Новая строка флагов
 */
string ModHostFlags(const string &flags, char flag, bool op);

/// Разделяемый указатель на объект CALDHost
typedef shared_ptr<CALDHost> CALDHostPtr;

/*! \class CALDAdmGroup
 * \brief Класс ALD - группа администраторов
 */
class CALDAdmGroup: public CALDObject
{
protected:
    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual bool ProcessEntity(bool quiet = false);
    /// Возвращает список членов (LDAP DN)
    void GetLDAPMembers(ald_string_list &memblist);
public:
    /// Конструктор объекта
    explicit CALDAdmGroup(CALDConnectionPtr Conn);
    /// Возвращает список членов
    void members(ald_string_list &userlist);
    /// Проверка на членство
    bool IsMember(const string &user);
	virtual bool Get(const string &agroup, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Добавление члена группы
    void AddMember(const string &user);
    /// Удаление члена группы
    void RmMember(const string &user);
    virtual bool CheckRemoval(bool quiet = false);
    void Remove(bool force);
    virtual ~CALDAdmGroup();
};

/// Разделяемый указатель на объект CALDAdmGroup
typedef shared_ptr<CALDAdmGroup>CALDAdmGroupPtr;

/*! \name Идентификаторы полей шаблонов протоколирования */
///@{
#define ALD_LOG_CFG_DEST        0
#define ALD_LOG_CFG_TPL         1
///@}

/*! \class CALDDomainLogCfg
 * \brief Класс ALD - шаблоны протоколирования
 */
class CALDDomainLogCfg: public CALDObject
{
protected:
    string m_strLogDest;    ///< Расположение файла журнала
    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual bool ProcessEntity(bool quiet = false);
public:
    /// Конструктор объекта
    explicit CALDDomainLogCfg(CALDConnectionPtr Conn);
    /// Возвращает расположение файла журнала
    string log_dest() const;
    /// Возвращает список шаблонов протоколирования
    void templates(ald_string_list &tpllist);
    /// Возвращает шаблон протоколирования по имени
    string GetTemplate(const string &tpl_name);
    /// Модификация шаблон протоколирования по имени и маске
    void ModTemplate(unsigned int setmask, const string &tpl_name, const string &suffix,
                     const string &mode);
    /// Удаление шаблона протоколирования по имени
    void RmTemplate(const string &tpl_name);
    /// Перегружаемая функция обновления объекта
    virtual void Update(const string &newdest);
    virtual bool Get(const string &oname = "", ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual bool CheckRemoval(bool quiet = false);
    void Remove(bool force);
    virtual ~CALDDomainLogCfg();
};

/// Разделяемый указатель на объект CALDDomainLogCfg
typedef shared_ptr<CALDDomainLogCfg>CALDDomainLogCfgPtr;

/*! \name Уровни информации доверенного домена */
///@{
// name, kdc, desc
#define ALD_TD_INFO_0   0
// krb
#define ALD_TD_INFO_1   1
///@{

/*! \name Идентификаторы полей доверенного домена */
///@{
#define ALD_TD_PROP_NAME        0
#define ALD_TD_PROP_KDC         1   // ALD_SET_NAME
#define ALD_TD_PROP_DESC        2   // ALD_SET_DESC
#define ALD_TD_PROP_DIR         3   // ALD_SET_FLAGS
///@{

/*! \name Типы доверительных отношений */
///@{
#define ALD_TD_DISABLED     0
#define ALD_TD_INBOUND      0x1 // Reserved
#define ALD_TD_OUTBOUND     0x2 // Reserved
#define ALD_TD_BIDIRECTIONAL (ALD_TD_INBOUND|ALD_TD_OUTBOUND) // default
///@}

/*! \class CALDTrustedDomain
 * \brief Класс ALD - доверенный домен
 */
class CALDTrustedDomain: public CALDObject
{
protected:
    string m_strKDC;                ///< Имя контроллера домена
    unsigned int m_uDirection;      ///< Тип отношений
    CALDKrbPrincipalPtr m_krbInTGT; ///< InTGT принципал Kerberos
    CALDKrbPrincipalPtr m_krbOutTGT;///< OutTGT принципал Kerberos
    string m_strInTGT;              ///< Имя InTGT принципала Kerberos
    string m_strOutTGT;             ///< Имя OutTGT принципала Kerberos
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual void InternalCreate(void *extra = NULL);
    virtual void InternalRemove(bool force, void *extra = NULL);

    virtual void InternalSave(ostream &os, bool verbose = false){ /*dont save, must be recreate */ }
    virtual void EnsureEntity();
    virtual bool ProcessEntity(bool quiet = false);
    void InternalSetName(const string &domain); ///< Синтез имен принципалов по домену
    virtual void InternalUpdate(unsigned int setmask, ald_string_multimap &attrs, void *extra = NULL);
public:
    /// Конструктор объекта
    explicit CALDTrustedDomain(CALDConnectionPtr Conn);

    /// Возвращает описание
    string desc();
    /// Возвращает имя контроллера домена
    string const &kdc() const;
    /// Возвращает имя InTGT принципала
    string const &strInTGT();
    /// Возвращает описание
    /// Возвращает имя OutTGT принципала
    string const &strOutTGT();
    /// Возвращает тип отношений
    unsigned int direction() const;
    /// Возвращает описание типа отношений
    string strDirection() const;
    /// Возвращает тип InTGT принципал
    CALDKrbPrincipalPtr inTGT();
    /// Возвращает тип OutTGT принципал
    CALDKrbPrincipalPtr outTGT();
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &domain, const string &newkdc, const string &newdesc,
                        unsigned int newdir, const string &newpwd);
    virtual bool Get(const string &domain, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &newkdc, const string &newdesc,
                        unsigned int newdir);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    virtual ~CALDTrustedDomain();
};

/// Разделяемый указатель на объект CALDTrustedDomain
typedef shared_ptr<CALDTrustedDomain> CALDTrustedDomainPtr;

/*! \name Уровни информации документа */
///@{
/// name, version, location
#define ALD_DOCUMENT_INFO_0   0
/// all
#define ALD_DOCUMENT_INFO_1   1
///@}

/*! \name Идентификаторы полей документа */
///@{
#define ALD_DOCUMENT_PROP_NAME      0  // ALD_SET_NAME
#define ALD_DOCUMENT_PROP_VER       1  // ALD_SET_STATE
#define ALD_DOCUMENT_PROP_TITLE     2  // ALD_SET_GECOS
#define ALD_DOCUMENT_PROP_LOCATION  3  // ALD_SET_FULL_NAME
#define ALD_DOCUMENT_PROP_DESC      4  // ALD_SET_DESC
///@}

/*! \name Идентификаторы внутренних документов */
///@{
#define ALD_INTDOC_UID  "ALD/DOMAIN/NextUID"
#define ALD_INTDOC_GID  "ALD/DOMAIN/NextGID"
#define ALD_INTDOC_TID  "ALD/DOMAIN/NextTID"
#define ALD_INTDOC_EID  "ALD/DOMAIN/NextEID"
///@}

/*! \class CALDDocument
 * \brief Класс ALD - документ
 */
class CALDDocument: public CALDObject
{
protected:
    string m_strVersion;    ///< Версия документа
    string m_strLocation;   ///< Расположение документа

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false);
    virtual bool ProcessEntity(bool quiet = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
public:
    /// Конструктор объекта
    explicit CALDDocument(CALDConnectionPtr Conn);
    /// Возвращает версию
    string const &version();
    /// Возвращает расположение
    string const &location();
    /// Возвращает заголовок
    string title();

    /// Возвращает описание
    string desc();
    /// Возвращает признак внутреннего документа
    bool IsInternal();
    virtual void Clear();
    /// Перегружаемая функция создания объекта по базовым параметрам
    virtual void Create(const string &newname, const string &newtitle, const string &newlocation,
                        const string &newver, const string &newdesc);
    virtual bool Get(const string &name, ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &newtitle, const string &newlocation,
                        const string &newver, const string &newdesc);
    virtual bool CheckRemoval(bool quiet = false);
    virtual void Remove(bool force = false);
    virtual bool ValidateProperty(int property, const void *value, string *msg, bool quiet);
    /// Сохранение документа на сервер
    virtual void Upload(const string &filename, bool overwrite);
    /// Загрузка документа с сервера
    virtual void Download(const string &filename = "");
    virtual ~CALDDocument();
};

/// Разделяемый указатель на объект CALDDocument
typedef shared_ptr<CALDDocument> CALDDocumentPtr;

/*! \name Идентификаторы полей домена */
///@{
#define ALD_DOMAIN_PROP_ORG      0  // ALD_SET_NAME
#define ALD_DOMAIN_PROP_STATE    0  // ALD_SET_STATE
#define ALD_DOMAIN_PROP_TEL      0  // ALD_SET_GECOS
#define ALD_DOMAIN_PROP_ADDR     0  // ALD_SET_FULL_NAME
#define ALD_DOMAIN_PROP_DESC     0  // ALD_SET_DESC
///@}

/*! \class CALDDomain
 * \brief Класс ALD - домен
 */
class CALDDomain: public CALDObject
{
protected:
    string m_strVersion; ///< Версия домена

    virtual void InternalSave(ostream &os, bool verbose = false);
    virtual aoLoadStatus InternalLoad(istream &is, size_t &nline, string &line, bool verbose = false);
    virtual void InternalNotice(aoOperation op, const string &arg, const string &detail);
    virtual void InternalGet(string object, ald_info_level level, bool quiet = false){ /* nothing to do */}
public:
    /// Конструктор объекта
    explicit CALDDomain(CALDConnectionPtr Conn);

    /// Возвращает версию домена
    string const &version() const;

    /// Возвращает организацию
    string org();

    /// Возвращает округ
    string state();

    /// Возвращает телефон
    string telephon();

    /// Возвращает адрес
    string address();

    /// Возвращает описание
    string desc();

    virtual void Clear();
    virtual bool Get(const string &domain = "", ald_info_level level = ALD_INFO_DEFAULT, bool quiet = false);
    /// Перегружаемая функция обновления объекта по базовым параметрам с указанием маски изменений
    virtual void Update(unsigned int setmask, const string &neworg, const string &newstate,
                        const string &newtel, const string &newaddr, const string &newdesc);
    // ----- groups
    /*! \brief Получение списка групп
     * @param[out] groups Список групп
     */
    void EnumerateGroups(ald_string_list &groups);
    /// Получение следующего GID (если указан generate - продвигает)
    gid_t NextGID(bool generate);

    /*! \brief Создание группы
     * @param[in] group Имя
     * @param[in] desc Описание
     * @param[in] users Список членов группы
     * @param[in] gid GID
     */
    void AddGroup(const string &group, const string &desc, const ald_string_list &users, gid_t gid = ALD_INVALID_ID);

    /*! \brief Поиск группы по имени или gid
     * @param[in] group имя или gid
     * @param[out] groupname Имя
     * @param[out] gid GID
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveGroup(const string &group, string &groupname, string &gid, bool quiet = false);

    // ----- users
    /*! \brief Получение списка пользователя
     * @param[out] users Список пользователей
     */
    void EnumerateUsers(ald_string_list &users);
    /// Получение следующего UID (если указан generate - продвигает)
    uid_t NextUID(bool generate);

    /*! \brief Создание пользователя
     * @param[in] user Имя
     * @param[in] pwd Пароль
     * @param[in] hometype Тип домашнего каталога
     * @param[in] home Путь к домашнему каталогу
     * @param[in] homesrv Сервер домашних каталогов
     * @param[in] shell Оболочка
     * @param[in] gecos GECOS
     * @param[in] desc Описание
     * @param[in] full_name Полное имя
     * @param[in] policy Политика
     * @param[in] lgroups Список локальных групп
     * @param[in] gid GID первичной группы
     * @param[in] uid UID
     * @param[in] needpwchange Признак смены пароля при входе
     */
    void AddUser(const string &user, const string &pwd, const string &hometype, const string &home,
                 const string &homesrv, const string &shell, const string &gecos, const string &desc,
                 const string &full_name, const string &policy,  const string &lgroups,
                 gid_t gid = ALD_INVALID_ID, uid_t uid = ALD_INVALID_ID, bool needpwchange = false);

    /*! \brief Поиск пользователя по имени или gid
     * @param[in] user имя или uid
     * @param[out] username Имя
     * @param[out] uid UID
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveUser(const string &user, string &username, string &uid, bool quiet = false);
    // ------ privileged users
    /*! \brief Получение списка привилегированных пользователя
     * @param[out] users Список пользователей
     */
    void EnumeratePrivilegedUsers(ald_string_list &users);

    // ----- tasks
    /*! \brief Получение списка заданий
     * @param[out] tasks Список заданий
     * @param[in] status_filter маска статусов заданий
     */
    void EnumerateTasks(ald_string_list &tasks, unsigned int status_filter);

    /*! \brief Получение списка заданий с указанного времени
     * @param[out] tasks Список заданий
     * @param[in] status_filter маска статусов заданий
     * @param[in] since метка времени
     */
    void EnumerateTasksSince(ald_string_list &tasks, unsigned int status_filter, time_t since);

    /// Получение следующего TID (если указан generate - продвигает)
    unsigned int NextTID(bool generate);

    /*! \brief Создание задания
     * @param[in] TaskType Тип задания
     * @param[in] TaskData Данные задания
     */
    string CreateTask(const string &TaskType, const string &TaskData);

    /// Ротация - удаление устаревших завершенных заданий
    void RotateTasks();

    // ----- events
    /*! \brief Получение списка событий
     * @param[out] events Список событий
     */
    void EnumerateEvents(ald_string_list &events);

    /*! \brief Получение списка событий с указанного времени
     * @param[out] events Список событий
     * @param[in] since метка времени
     */
    void EnumerateEventsSince(ald_string_list &events, time_t since);

    /// Получение следующего EID (если указан generate - продвигает)
    unsigned int NextEID(bool generate);

    /*! \brief Создание события
     * @param[in] TaskType Тип события
     * @param[in] TaskData Данные задания
     */
    string RaiseEvent(const string &TaskType, const string &TaskData);

    /// Ротация - удаление устаревших событий
    void RotateEvents();

    // ----- policies
    /*! \brief Получение списка политик
     * @param[out] policies Список политик
     */
    void EnumeratePolicies(ald_string_list &policies);

    /// Создание политики
    void AddPolicy(const string &policy, unsigned int minlife, unsigned int maxlife, unsigned int minlength,
                   unsigned int minclasses, unsigned int historynum, unsigned int newmaxfail,
                   unsigned int newfailcntinterval, unsigned int newlockoutduration);

    /*! \brief Поиск политики по имени
     * @param[in] policy Имя
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolvePolicy(const string &policy, bool quiet = false);

    // ----- Services
    /*! \brief Получение списка сервисов
     * @param[out] services Список сервисов
     */
    void EnumerateServices(ald_string_list &services);

    /// Создание сервиса
    void AddService(const string &service);

    /*! \brief Поиск сервиса по имени
     * @param[in] service Имя
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveService(const string &service, bool quiet = false);

    // ----- Services groups
    /*! \brief Получение списка групп сервисов
     * @param[out] sgroups Список групп сервисов
     */
    void EnumerateServicesGroups(ald_string_list &sgroups);

    /*! \brief Поиск группы сервисов по имени
     * @param[in] sgroup Имя
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveServicesGroup(const string &sgroup, bool quiet = false);

    // ----- Hosts
    // ----- Services groups
    /*! \brief Получение списка компьютеров
     * @param[out] hosts Список компьютеров
     */
    void EnumerateHosts(ald_string_list &hosts);

    /// Создание компьютера
    void AddHost(const string &host, const string &desc);

    /*! \brief Поиск компьютера по имени с нормализацией
     * @param[in] host Имя
     * @param[out] hostname Нормализованное имя
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveHost(const string &host, string &hostname, bool quiet = false);

    // ----- Hosts groups
    /*! \brief Получение списка групп компьютеров
     * @param[out] hgroups Список групп компьютеров
     */
    void EnumerateHostsGroups(ald_string_list &hgroups);

    /// Создание группы компьютеров
    void AddHostsGroup(const string &hgroup, const string &hgroupdesc);

    /*! \brief Поиск группы компьютеров по имени
     * @param[in] hgroup Имя
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveHostsGroup(const string &hgroup, bool quiet = false);

    // ----- Tasks
    /// Возвращает размер историии заданий
    unsigned int taskHistory();

    /// Устанавливает размер историии заданий
    void SetTaskHistory(unsigned int value);

    /// Обновляет метку времени сервера
    void UpdateServerTimestamp();

    /// Возвращает метку времени сервера
    string server_timestamp();

    // ----- FileServers
    /*! \brief Получение списка файлсерверов
     * @param[out] fileservers Список файлсерверов
     */
    void EnumerateFileServers(ald_string_list &fileservers);

    /*! \brief Получение списка серверов домашних каталогов
     * @param[out] homeservers Список серверов домашних каталогов
     */
    void EnumerateHomeServers(ald_string_list &homeservers);

    // --------------------
    /*! \brief Получение списка серверов ALD
     * @param[out] servers Список серверов ALD
     */
    void EnumerateALDServers(ald_string_list &servers);

    /*! \brief Поиск ALD сервера по имени или идентификатору
     * @param[in,out] host Имя компьютера
     * @param[in,out] server_id Контроллер домена
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveALDServer(string &host, unsigned int &server_id, bool quiet = false);

    // ----- TrustedDomains
    /*! \brief Получение списка доверенных доменов
     * @param[out] realms Список доверенных доменов
     */
    void EnumerateTrustedDomains(ald_string_list &realms);

    /// Создание довереного домена
    void AddTrustedDomain(const string &realm, const string &kdc, const string &desc,
                          unsigned int newdir, const string &newpwd);

    /*! \brief Поиск доверенного домена
     * @param[in] realm Имя домена
     * @param[out] kdc Контроллер домена
     * @param[in] quiet Флаг тихого режима
     * @return true - если найдено
     */
    bool ResolveTrustedDomain(const string &realm, string &kdc, bool quiet = false);

    // ----- Documents
    /*! \brief Получение списка документов
     * @param[out] docs Список документов
     * @param[in] internal_oly Признак
     */
    void EnumerateDocuments(ald_string_list &docs, bool internal_oly);

    /// Плучение спсика внешних документов, созданных начиная с since
    void EnumerateDocumentsSince(ald_string_list &docs, time_t since);

    /// Создание внешнего документа
    void AddDocument(const string &name, const string &title, const string &location,
                     const string &ver, const string &desc);

    /// Создание внутреннего документа
    void AddInternalDocument(const string &name, const string &ver, const string &desc);

    // serialization
    /// Сохранение домена в переносимом текстовом формате в указанный файл
    virtual void Save(const string &pbkfile, bool verbose = false);
    /// Сохранение домена в переносимом текстовом формате в указанный поток вывода
    virtual void Save(ostream &os, bool verbose = false);
    /// Восстановление домена в переносимом текстовом формате из указанного файла
    virtual void Load(const string &pbkfile, bool verbose = false);
    /// Восстановление домена в переносимом текстовом формате из указанного потока ввода
    virtual void Load(istream &is, const string &bkname, bool verbose = false);

    virtual bool CheckRemoval(bool quiet = false);

    void Remove(bool force);

    // ----- Common
    /*! \brief Удаление объекта домена ALD
     * @param[in] otype Тип объекта
     * @param[in] oname Имя объекта
     * @param[in] checkonly Только проверка удаления
     * @return Результат выполнения удаления
     */
    virtual bool RemoveObject(const string &otype, const string &oname, bool checkonly);

    virtual ~CALDDomain();
};

/// Разделяемый указатель на объект CALDDomain
typedef shared_ptr<CALDDomain> CALDDomainPtr;

/// Возвращает разделяемый указатель на объект комьютер для сервера
CALDHostPtr GetServer(CALDConnectionPtr Conn);
/// Возвращает разделяемый указатель на объект комьютер для текущего хоста
CALDHostPtr GetHost(CALDConnectionPtr Conn);

/*! \brief Обновление ключей сервиса в ключевом файле
* @param[in] Conn Разделяемый указатель на соединение
* @param[in] keytab Путь к файлу ключей
* @param[in] service Имя сервиса
* @param[in] create Флаг создания сервиса при отсутствии
* @return Успех или отказ
*/
bool UpdateServiceInKeytab(CALDConnectionPtr Conn, const string &keytab, const string &service, bool create);

/// Печать операционной информации LDAP по указанному объекту в заданный поток вывода
void ShowLdapOperInfo(CALDObject &object, ostream &out);

/*! \brief Установка именнованной метки времени объекта
* @param[in] object Объект
* @param[in] tsname Имя метки
* @param[in] ts Метка времени
*/
void SetALDTimestamp(CALDObject &object, const string &tsname, time_t ts = 0); // 0 - now

/// Возвращает текстовое представление именованной метки времени объекта
string GetALDTimestamp(CALDObject &object, const string &tsname);

/// Структура событий объекта
struct ald_object_action_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    // cppcheck-suppress unusedStructMember
    bool quiet;             ///< Признак тихого режима
    // cppcheck-suppress unusedStructMember
    void *extra;            ///< Доп. параметр
};

/*! \name Структуры передачи информации при генерации событий */
///@{
/// Структура данных событий PAM модуля (событие вида ALD_PAM_*)
struct ald_pam_user_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    string user;            ///< Пользователь
    string home;            ///< Домашний каталог
    string hometype;        ///< Тип домашнего каталога
    string homeserver;      ///< Сервер домашнего каталога
    uid_t uid;              ///< UID
    gid_t gid;              ///< GID
    ald_string_list mounts; ///< Список монтируемых каталогов
    unsigned int ctrls;     ///< Опции PAM
    // cppcheck-suppress unusedStructMember
    void *pamh;             ///< Дескриптор PAM сессии
    // cppcheck-suppress unusedStructMember
    void *extra;            ///< Доп. параметр
    //
    IALDCore* pCore;
    bool bClientOn;
    bool bLoggedInCached;
    bool bHostDisabled;
    bool bUserLocked;
    bool bUserPermittedToHost;
    int  ngids;
    gid_t *pgids;
};

/// Структура данных сохранения резервной копии (событие ALD_PORTABLE_SAVE)
struct ald_portable_save_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    string otype;           ///< Тип объекта
    ostream *os;            ///< Указатель потока вывода
    // cppcheck-suppress unusedStructMember
    bool verbose;           ///< Признак вывода информации
    // cppcheck-suppress unusedStructMember
    void *extra;            ///< Доп. параметр
};

/// Структура данных загрузки резервной копии (событие ALD_PORTABLE_LOAD)
struct ald_portable_load_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    string version;         ///< Версия
    istream *is;            ///< Указатель потока ввода
    // cppcheck-suppress unusedStructMember
    size_t nline;           ///< Номер строки
    // cppcheck-suppress unusedStructMember
    bool loaded;            ///< Признак успеха загрузки
    // cppcheck-suppress unusedStructMember
    bool verbose;           ///< Признак вывода информации
    // cppcheck-suppress unusedStructMember
    void *extra;            ///< Доп. параметр
};


/// Структура данных удаления объекта (событие ALD_DELETE_OBJECT)
struct ald_delete_object_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    string otype;           ///< Тип объекта
    // cppcheck-suppress unusedStructMember
    bool checkonly;         ///< Только проверка без удаления
    // cppcheck-suppress unusedStructMember
    bool deleted;           ///< Признак успеха удаления
    // cppcheck-suppress unusedStructMember
    void *extra;            ///< Доп. параметр
};

/// Структура данных обработки заданий (событие ALD_TASK_PENDING)
struct ald_task_extra {
    CALDConnectionPtr conn; ///< Разделяемый указатель на соединение
    string id;              ///< Идентификатор задания
    string type;            ///< Тип задания
    string data;            ///< Данные задания
    string error;           ///< Сообщение об ошибке
    bool done;              ///< Признак успеха выполнения
    bool skip;              ///< Признак пропуска задания
    void *extra;            ///< Доп. параметр
};
///@}

/*! \name Группа вспомогательных макросов */
///@{
#define CHECK_VALID ALD_CHECK(m_bValid, ERR_INVALID_OBJECT)
#define CHECK_NAME_VALID ALD_CHECK(!m_strName.empty(), ERR_INVALID_OBJECT)
#define QLOG_WARN(warn) {if(!quiet) LOG_WARN(warn);}
#define QLOG_ERR(warn) {if(!quiet) LOG_ERR(warn);}
#define QALD_ERR(err) {if(!quiet) ALD_ERROR(err);}
///@{

/*! \name Группа макросов вызовов триггеров объекта */
///@{
#define ALD_OBJECT_TRIGGER_CHECK(action, quiet)  TriggerAction(ALD_OBJECT_ACTION(PFM2Name(__PRETTY_FUNCTION__), \
    it->first), aotatCheck, quite)
#define ALD_OBJECT_TRIGGER_ACTION(action, quiet)  TriggerAction(ALD_OBJECT_ACTION(PFM2Name(__PRETTY_FUNCTION__), \
    it->first), aotatAction, quite)
///@{

/*! \name Группа макросов нотификации операций с объектом */
///@{
#define ALD_OBJECT_START_OPERATION(op, arg) StartOperation(PFM2Name(__PRETTY_FUNCTION__), op, arg, "", false)
#define ALD_OBJECT_START_OPERATION_FORCE(op, arg, force) StartOperation(PFM2Name(__PRETTY_FUNCTION__), op, arg, "", force)
#define ALD_OBJECT_START_OPERATION_DETAIL(op, arg, det) StartOperation(PFM2Name(__PRETTY_FUNCTION__), op, arg, det, false)
#define ALD_OBJECT_COMMIT_OPERATION(op) CommitOperation(PFM2Name(__PRETTY_FUNCTION__), op, false)
#define ALD_OBJECT_COMMIT_OPERATION_PE(op) CommitOperation(PFM2Name(__PRETTY_FUNCTION__), op, true)
#define ALD_OBJECT_COMMIT_OPERATION_PEQ(op) CommitOperation(PFM2Name(__PRETTY_FUNCTION__), op, true, quiet)
///@{

/// Проверка возможности управления компьютером указанным администратором
bool IfAdminCanManageHost(CALDConnectionPtr conn, const string& admin, const string& host);

/// Проверка возможности управления группой компьютеров указанным администратором
bool IfAdminCanManageHostGroup(CALDConnectionPtr conn, const string& admin, const string& hostgroup);

///@}
} // ALD

#endif // ALD_OBJECTS
