/*! \file ALDRpcUtils.h
 * \brief Объявление вспомогательных типов и функций RPC Astra Linux Directory
 * \ingroup ALDRPCUtils_group
 */

#ifndef ALD_RPC_UTILS
#define ALD_RPC_UTILS

#include "common.h"
#include <iostream>
#include <memory>
#include "ald-rpc.h"
#include "rpc-definitions.h"

using namespace std;

namespace ALD
{

/*! \defgroup ALDRPCUtils_group Вспомогательные типы и функции RPC (ALDRpcUtils.h)
 * \brief Объявление вспомогательных типов и функций RPC Astra Linux Directory
 * \ingroup IALDRpcWrapper_group
 * @{
 */

/// Запись строки в поток
void send_str(ostream& o, const string& s, bool checklen = true);
/// Чтение строки из потока
void read_str(istream& i, string& s, bool checklen = true);

/// Запись размера в поток
void send_size(ostream& o, size_t s);
/// Чтение размера из потока
void read_size(istream& i, size_t& s);

class IALDRpcSession;
class CALDUser;

// Privileged     everybody   authenticated     authorized(CREDS)   admins     admin/admin
/// Класс безопасности команды
enum arcPrivKind{
    arckPublic,         ///< Публичная
    arckAuthenticated,  ///< Для аутентифицированных пользователей
    arckAuthorized,     ///< Для авторизованных пользователей
    arckPrivileged,     ///< Для привилегированных пользователей
    arckSystem};        ///< Системная

/*! \class CALDRpcCommand
 * \brief Класс RPC команды
 */
class CALDRpcCommand
{
protected:
    IALDCore* m_pCore;      ///< Указатель на интерфейс ядра
    string m_strName;       ///< Имя
    string m_strDesc;       ///< Описание
    arcPrivKind m_PrivKind; ///< Класс безопасности (default arckPrivileged)
public:
    /// Возвращает имя
    inline const string &name() const {return m_strName;}
    /// Возвращает описание
    inline const string &desc() const {return m_strDesc;}
    /// Возвращает класс безопасности
    inline arcPrivKind kind() const {return m_PrivKind;}

    /*! \brief Перегружаемая процедура обработки
     * @param[in] request RPC запрос
     * @param[in] session Указатель на интерфейс RPC сессии
     * @param[in] extra Доп. параметр
     */
    virtual void Run(ald_rpc_request &request, IALDRpcSession *session, void* extra) = 0;

    /// Конструктор
    CALDRpcCommand(IALDCore& Core);
    virtual ~CALDRpcCommand();
};

/// Разделяемый указатель на RPC команду
typedef shared_ptr<CALDRpcCommand> CALDRpcCommandPtr;

/// Карта RPC команд (имя => разделяемый указатель на RPC команду)
typedef map<string, CALDRpcCommandPtr> ald_rpc_command_map;

/*! \class IALDRpcSession
 * \brief Абстрактный интерфейс RPC сессии
 */
class IALDRpcSession
{
public:
    virtual ald_rpc_request& request() = 0; ///< Возвращает RPC запрос
    /*! \name Возвращает признаки авторизации */
    ///@{
    virtual bool IsAuthenticated() = 0;
    virtual bool IsAuthorized() = 0;
    virtual bool IsAdmin() = 0;
    virtual bool IsSuperuser() = 0;
    virtual bool IsService() = 0;
    ///@}
    virtual string userId() = 0;            ///< Возвращает идентификатор клиента
    /*! \name Возвращает адресную информацию */
    ///@{
    virtual string localhost() = 0;
    virtual int localport() = 0;
    virtual string peerhost() = 0;
    virtual int peerport() = 0;
    ///@}

    /// Возвращает указанную карту команд (RPC или LPC)
    virtual ald_rpc_command_map& commands(const string &key) = 0;

    virtual ~IALDRpcSession(){}
};

/* Actions */
/*! \name Идентификаторы RPC событий */
///@{
#define ALD_LOAD_RPC_COMMANDS	"LoadRpcCommands"
#define ALD_RPC_KEY         	"RPC"
#define ALD_LPC_KEY         	"LPC"
///@}

/// Структура события ALD_LOAD_RPC_COMMANDS
struct ald_load_rpc_commands_extra {
    ald_rpc_command_map* mapRpcCmds;    ///< Карта RPC команд
    // cppcheck-suppress unusedStructMember
    void *extra;                        ///< Доп. параметр
};

/// Вовзращает описание типа сообщения
const char* rpcmsgtype2str(unsigned short message);

/// Структура события ALD_STATISTICS
struct ald_rpc_statistics_extra {
    ald_string_map *stats;      ///< Карта счетчиков статистики
    // cppcheck-suppress unusedStructMember
    void *extra;                ///< Доп. параметр
};

/// Создание имени широковещательного сообщения
#define MK_BROADCAST_MESSAGE(msg, arg) string(msg)+":"+arg

/// Отправка широковещательного сообщения
bool SendBroadcastMessage(IALDCore &Core, const string& msg, ald_string_list *responses = NULL, int timeout = 3);

///@}

} // ALD

#endif //ALD_RPC_UTILS
