/*! \file ALDUtils.h
 * \brief Объявление вспомогательных типов и функций общего назначения
 * \ingroup ALDUtils_group
 */

#ifndef ALD_UTILS
#define ALD_UTILS

#include <sys/stat.h>
#include <sys/capability.h>
#include <string>
#include <list>
#include <map>
#include <vector>
#include <cstdint>
#include <ctime>

using namespace std;

namespace ALD
{

/*! \defgroup ALDUtils_group Типы и функции общего назначения (ALDUtils.h)
 * \brief Объявление вспомогательных типов и функций общего назначения
 * @{
 */

/// Макрос форматирования
#define FMTSTR CALDFormatCall(__FILE__, __FUNCTION__, __LINE__)

// terminal codes
/*! \name Макросы работы с управляющими кодами терминала */
///@{
#define T_CMD(list) "\x1b[" list "m"
#define T_DFLT "\x1b[0;m"

// styles
#define T_NORM      "0"
#define T_BOLD      "1"
#define T_UNDE      "4"
#define T_BLINK     "5"
#define T_INVERT    "7"

// font color
#define T_FBLACK    "30"
#define T_FRED      "31"
#define T_FGREEN    "32"
#define T_FYELLOW   "33"
#define T_FBLUE     "34"
#define T_FMAGENTA  "35"
#define T_FCYAN     "36"
#define T_FWHITE    "37"

// backgraund color
#define T_BBLACK    "30"
#define T_BRED      "31"
#define T_BGREEN    "32"
#define T_BYELLOW   "33"
#define T_BBLUE     "34"
#define T_BMAGENTA  "35"
#define T_BCYAN     "36"
#define T_BWHITE    "37"
///@}

// terminal color command
/// Устанавливает признак использования цветового офрмления в терминале
void UseTermColors(bool colored);
/// Возвращает признак использования цветового офрмления в терминале
bool IsTermColored();

/// Условный вывод команды форматирования терминала
string t_cmd(const char * cmd);
/// Условный вывод команды сброса форматирования терминала
string t_dflt();

/*! \name Макросы работы с цветовым офрмлением терминала */
///@{
#define BRED        t_cmd(T_BOLD ";" T_FRED)
#define BGREEN      t_cmd(T_BOLD ";" T_FGREEN)
#define BYELLOW     t_cmd(T_BOLD ";" T_FYELLOW)
#define BBLUE       t_cmd(T_BOLD ";" T_FBLUE)
#define BMAGENTA    t_cmd(T_BOLD ";" T_FMAGENTA)
#define BCYAN       t_cmd(T_BOLD ";" T_FCYAN)
#define BWHITE      t_cmd(T_BOLD ";" T_FWHITE)

#define RED         t_cmd(T_FRED)
#define GREEN       t_cmd(T_FGREEN)
#define YELLOW      t_cmd(T_FYELLOW)
#define BLUE        t_cmd(T_FBLUE)
#define MAGENTA     t_cmd(T_FMAGENTA)
#define CYAN        t_cmd(T_FCYAN)
#define WHITE       t_cmd(T_FWHITE)

#define NORMAL t_dflt()

#define OUT_COLOR(out, msg, color) do{out << color; out << msg; out << NORMAL; out << endl;}while(0)
#define COUT_COLOR(msg, color) OUT_COLOR(cout, msg, color)
#define COUT_ERR(msg) COUT_COLOR(msg, BRED)
#define COUT_WARN(msg) COUT_COLOR(msg, YELLOW)
#define COUT_CHECK(msg) COUT_COLOR(msg, GREEN)
#define COUT_INFO(msg) COUT_COLOR(msg, CYAN)
#define COUT_TEST(msg) COUT_COLOR(msg, MAGENTA)
///@}

/*! \class CALDFormatCall
 * \brief Автоматический класс форматного вывода с валидацией количества параметров
 */
class CALDFormatCall
{
protected:
    string m_strBuff;   ///< Внутренний строковый буфер
    const char* m_pszFile;  ///< Имя файла
    const char* m_pszFunc;  ///< Имя функции
    unsigned long m_uLine;  ///< Номер строки
public:
    /*! \brief Оператор () для форматного вывода с переменным кол-вом аргументов
     * @param[in] nparams Количество параметров
     * @param[in] format Форматная строка
     * @param[in] ... Параметры
     */
    const char* operator()(size_t nparams, const char *format, ...);
    /*! \brief Конструктор класса форматного вывода
     * @param[in] file Имя файла
     * @param[in] func Имя функции
     * @param[in] line Номер строки
     */
    CALDFormatCall(const char* file, const char* func, unsigned long line);
    /// Деструктор
    virtual ~CALDFormatCall();
};

/// Возвращает имя файла по полному пути
const char *ExtractFileName(const char* pszFileName);

/// Возвращает короткое имя функции по полному имени
const char *ExtractFuncName(const char* pszFuncName);

/*! \class CALDStaticHelper
 * \brief Статический класс помошник
 */
class CALDStaticHelper
{
public:
    /// Конструктор
    CALDStaticHelper();
    /// Деструктор
    virtual ~CALDStaticHelper();
};

// ald types definitions
/// Список строк
typedef list<string>      ald_string_list;
/// Список целых
typedef list<int>         ald_int_list;

/// Вектор строк
typedef vector<string>    ald_string_vector;
/// Вектор целых
typedef vector<int>       ald_int_vector;

/// Пара строк
typedef pair<string, string>      ald_string_pair;
/// MAP строка => строка
typedef map<string, string>       ald_string_map;
/// MultiMAP строка => строка
typedef multimap<string, string>  ald_string_multimap;
/// MAP строка => число
typedef map<string, int>          ald_int_map;

// conversion functions
/*! \name Функции преобразования строк и целых */
///@{
bool str2i(const string &str, int &val);
string i2str(int val);
bool str2u(const string &str, unsigned int &val);
string u2str(unsigned int val);
bool hex2u(const string &str, unsigned int &val);
string u2hex(unsigned int val);
bool oct2u(const string &str, unsigned int &val);
string u2oct(unsigned int val);
bool strTou64(const string &str, uint64_t &val);
string u64Tostr(uint64_t val);
///@}

/*! \name Функции преобразования строк и типов времени */
///@{
string TimeToStr(time_t t, bool rfc2822);
time_t StrToTime(const string &strdt, bool rfc2822);

// GeneralizedTime = LDAP UTCTime syntax - 1.3.6.1.4.1.1466.115.121.1.53
time_t GeneralizedTimeToTime(const string &gt);
string GeneralizedTimeToStr(const string &gt, bool rfc2822);
string TimeToGeneralizedTime(time_t t);
///@}

// string functions
/*! \name Функции работы со строками */
///@{
string TrimLeft(const string &str);
string TrimRight(const string &str);
string Trim(const string &str);
string Unquote(const string &str);
string Quote(const string &str, bool force = false);
string Capitallize(const string &str);
string string2upper(const string &str);
string string2lower(const string &str);
string &StrAppend(string &str1, const string &str2);

char* strdup(const char* str);
char* strdup(const string &str);
///@}


// special string functions
/*! \name Функции преобразования имен доменов */
///@{
string DomainToDn(const string &domain);
string DomainToRealm(const string &domain);
string RemoveRealm(const string &str);
string DnToDomain(const string &dn);
string RealmToDomain(const string &realm);
string RemoveDomain(const string &str, const string &domain);
///@}

/// Разбор строки (имя=значение)
bool ParsePair(const string &line, string &name, string &value);

// conversation functions
/*! \name Функции запроса ответа у пользователя */
///@{
string AskPassword(const string &prompt, size_t size, char star);
string AskText(const string &prompt, size_t size);
string AskTextDef(const string &prompt, const string &defText, size_t size);
bool AskYesNo(const string &prompt, bool defYes);
unsigned int AskNumber(const string &prompt, unsigned int defVal);
///@}

// list functions
/*! \name Функции работы со списками строк */
///@{
void list_merge(ald_string_list &al1, const ald_string_list &al2);
void list_subtract(ald_string_list &al1, const ald_string_list &al2);
void list_intersect(ald_string_list &al1, const ald_string_list &al2);
bool IsMemberOfList(const ald_string_list &list, const string &str);
string listToStr(const ald_string_list &list);
void StrTolist(const string &str, ald_string_list &list, char delim = ',');
///@}

// FS functions
/*! \name Функции работы с путями и файлами */
///@{
bool IsFileExist(const string &path);
bool IsDirExist(const string &path);
bool IsPathExist(const string &path);
void RemoveDir(const string &path, bool force);
void RemoveFile(const string &path);
void RemovePath(const string &path, bool force);
void RemoveFiles(const string &path, const string &pattern);
void CreateDir(const string &path, mode_t mode);
void EnsureDir(const string &path, mode_t mode);
void Chown(const string &path, uid_t uid, gid_t gid, bool rec = false);
void Chown(const string &path, const string &ouser, const string &ogroup, bool rec = false);
void Chmod(const string &path, mode_t mode, bool rec = false);
bool IsDirectory(const string &path);
bool IsFile(const string &path);
void RenameFile(const string &file, const string &targetFile);
void CopyFile(const string &file, const string &targetFile);
void CopyPath(const string &path, const string &targetPath, mode_t mode);
void CreateFile(const string &file);
string GetFileName(const string &file);
string GetFilePath(const string &file);
string GetFileExt(const string &file);
///@}

// pack functions
/*! \name Функции работы со сжатием файлов */
///@{
void GZipFile(const string &file, const string &targetFile);
void GunZipFile(const string &file, const string &targetFile);

void TarCreate(const ald_string_list &files, const string &targetFile, int mode);
void TarExtract(const string &file, const string &targetPath, int mode);

void Pack(const ald_string_list &files, const string &targetFile, int mode);
void Unpack(const string &file, const string &targetPath, int mode);
///@}

/// Возвращает текущего пользователя
string CurrentUser();
/// Возвращает кодовую страницу по умолчанию
string DefaultCodeset();
/// Разыменовывает компьютер по имени
string HostName(const string &host);
/// Разворачивает домашний каталог (~)
string TildeSubst(const string &file);
/// Возвращает домен по имени компьютера
string HostDomain(const string &host);

// other defs
/// Максимальная длина имени компьютера
#define ALD_HOST_MAX_SIZE           255

/// Возвращает короткое имя функции по полному
string PFM2Name(const string &PFM);

// capabilities
/*! \name Работа с привилегиями Linux */
///@{
typedef uint32_t ald_linux_caps;
void EnumerateLinuxCaps(ald_string_list &caps);
ald_linux_caps LinuxCapByName(const string &capname);
enum aldCapFlag {acfEffective = 1, acfPermitted = 2, acfInheritable = 4};
typedef unsigned ald_cap_flags; // set of aldCapFlag
void LinuxCapMod(cap_value_t cap, ald_cap_flags flags, cap_flag_value_t val);
void LinuxCapsClear(ald_cap_flags flags);
///@}

// test integrity
/*! \name Макросы поддержки форматирования тестов целостности */
///@{
// variable bVerbose must be defined
#define TI_ITEM_INFO(info)  {if(bVerbose){TestIntegrityItem(info);}}
#define TI_SECTION_INFO(info) TestIntegritySection(info, bVerbose)
#define TI_OK        {if(bVerbose){TestIntegrityOK();}}
#define TI_ERROR     {if(bVerbose){TestIntegrityERROR();}}
#define TI_WARNING   {if(bVerbose){TestIntegrityWARNING();}}
#define TI_FIXED     {if(bVerbose){TestIntegrityFIXED();}}
#define TI_ITEM_RES(res)  {if(res) TI_OK else TI_ERROR;}
#define TI_SECTION_RES(res)  {if(!bVerbose) { cout << t_cmd(T_BOLD); res? TestIntegrityOK(): TestIntegrityERROR();}}

#define TI_ADD_ERRINFO(info, errlist) errlist.push_back(YELLOW+std::string(info)+NORMAL);
///@}

/*! \name Функции поддержки форматирования тестов целостности */
///@{
void TestIntegrityItem(const string& info);
void TestIntegritySection(const string& info, bool verb);
void TestIntegrityOK();
void TestIntegrityERROR();
void TestIntegrityWARNING();
void TestIntegrityFIXED();
///@}

///@}
} // ALD

#endif // ALD_UTILS
