/*! \file ald-config.h
 * \brief Объявление интерфейса конфигурирования Astra Linux Directory
 * \ingroup IALDConfig_group
 */

#ifndef ALD_CONFIG
#define ALD_CONFIG

#include "common.h"
#include "ALDCore.h"
#include <list>
#include <memory>
#include "gettext.h"

using namespace std;
using namespace ALD;

namespace ALD
{
/*! \defgroup IALDConfig_group Интерфейс конфигурирования (IALDConfig)
 * \brief Интерфейс работы с конфигурационными файлами
 * @{
 */

/// Имя интерфейса конфигурирования
#define ALD_CFG_ITF_NAME "ald-config"

class IALDConfig;

/*! \class IALDConfigFile
 * \brief Абстрактный класс-интерфейс работы с конфигурационным файлом
 * \details Предоставляет интерфейс чтения, сохранения и модификации конфигурационного файла
 */
class IALDConfigFile
{
protected:
    IALDCore* m_pCore; ///< Указатель на основной интерфейс клиента IALDCore
public:
    IALDConfigFile(IALDCore &Core){m_pCore = &Core;}  ///< Конструктор

    /*! \brief Открытие конфигурационного файла
     * @param[in] filename Путь к конфигурационному файлу
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[in] no_comment Флаг добавления ALD комментариев при сохранении
     * @return Успешность операции
     */
    virtual bool Open(const string &filename, const string& mode, bool no_comment = false) = 0;

    /*! \brief Открытие шаблона конфигурационного файла
     * @param[in] templatename Имя шаблона конфигурационного файла
     * @param[in] destfilename Путь к сохраняемому конфигурационному файлу
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[in] no_comment Флаг добавления ALD комментариев при сохранении
     * @return Успешность операции
     */
    virtual bool OpenTemplate(const string &templatename, const string &destfilename,
                              const string& mode, bool no_comment = false) = 0;

    /*! \brief Обработка шаблона с подстановкой значений переменных
     * @param[in] mapValues Таблица подстановочных переменных
     */
    virtual void ApplyTemplates(ald_string_map& mapValues) = 0;

    /*! \brief Удаление строк обрамленных заданными маркерами
     * @param[in] beginmarker Маркер начала (например ALD_BEGIN_MARKER)
     * @param[in] endmarker Маркер конца (например ALD_END_MARKER)
     */
    virtual void RemoveMarkers(const string &beginmarker, const string &endmarker) = 0;

    virtual void RemoveALDMarkers() = 0; ///< Удаление комментариев Astra Linux Directory

    /*! \brief Установка значения параметра по имени
     * @param[in] param Имя параметра
     * @param[in] value Значение параметра
     *  \details Предыдущее значение комментируется, если при открытии не был установлен флаг no_comment
     */
    virtual void SetParamValue(const string &param, const string &value) = 0;

    /*! \brief Получение значения параметра по имени
     * @param[in] param Имя параметра
     */
    virtual string GetParamValue(const string &param) = 0;

    /*! \brief Восстановление ранее закоментированного значения параметра по имени
     * @param[in] param Имя параметра
     */
    virtual void RestoreParamValue(const string &param) = 0;

    /*! \brief Установка строки по шаблону (регулярному выражению)
     * @param[in] token Шаблон (регулярное выражение)
     * @param[in] line Новое значение строки
     *  \details Предыдущее значение строки комментируется, если при открытии не был установлен флаг no_comment
     */
    virtual void SetLine(const string &token, const string &line) = 0;

    /*! \brief Получение строки по шаблону (регулярному выражению)
     * @param[in] token Шаблон (регулярное выражение)
     */
    virtual string GetLine(const string &token) = 0;

    /*! \brief Восстановление ранее закоментированного значения строки по шаблону (регулярному выражению)
     * @param[in] token Шаблон (регулярное выражение)
     */
    virtual void RestoreLine(const string &token) = 0;

    /*! \brief Сохранение конфигурационного файла
     * @param[in] no_reserve Не сохранять предыдущую версию конфигурационного файла
     */
    virtual void Save(bool no_reserve = false) = 0;

    virtual ~IALDConfigFile(){}  ///< Деструктор
};

typedef shared_ptr<IALDConfigFile> CALDConfigFilePtr;  ///< Разделяемый указатель на интерфейс работы с конфигурационным файлом IALDConfigFile

/*! \class IALDConfig
 * \brief Абстрактный класс-интерфейс конфигурирования Astra Linux Directory
 * \details Предоставляет интерфейс установки, настройки и чтения конфигурационных файлов
 */
class IALDConfig: public CALDInterface
{
public:
    IALDConfig(IALDCore &Core):CALDInterface(ALD_CFG_ITF_NAME, Core){}  ///< Конструктор

    /*! \brief Открытие конфигурационного файла
     * @param[in] filename Путь к конфигурационному файлу
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[in] no_comment Флаг добавления ALD комментариев при сохранении
     * @return Разделяемый указатель на интерфейс работы с конфигурационным файлом IALDConfigFile
     */
    virtual CALDConfigFilePtr OpenConfigFile(const string& filename, const string& mode,
                                             bool no_comment = false) = 0;

    /*! \brief Открытие шаблона конфигурационного файла
     * @param[in] templatename Имя шаблона конфигурационного файла
     * @param[in] destfilename Путь к сохраняемому конфигурационному файлу
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[in] no_comment Флаг добавления ALD комментариев при сохранении
     * @return Разделяемый указатель на интерфейс работы с конфигурационным файлом IALDConfigFile
     */
    virtual CALDConfigFilePtr OpenConfigFileTemplate(const string& templatename,
                                                     const string &destfilename,
                                                     const string& mode,
                                                     bool no_comment = false) = 0;

    virtual void RecreateTemplatesMap() = 0; ///< Инициализация таблица подстановочных переменных из системной таблицы переменных ALD
    virtual ald_string_map& TemplatesValues() = 0; ///< Таблица подстановочных переменных

    /*! \brief Установка шаблона конфигурацонного файла
     * @param[in] templatename Имя шаблона конфигурационного файла
     * \details Перенос шаблона в конфигурационный каталог ALD (восстановление исходного варианта)
     */
    virtual void InstallConfigFileTemplate(const string& templatename) = 0;

    /*! \brief Обработка шаблона конфигурацонного файла с подстановкой значений переменных
     * @param[in] templatename Имя шаблона конфигурационного файла
     * @param[in] destfilename Путь к сохраняемому конфигурационному файлу
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[in] pmapValues Указатель на дополнительную таблицу подстановочных переменных
     */
    virtual void ProcessConfigFileTemplate(const string& templatename, const string& destfilename,
                                           const string& mode, ald_string_map *pmapValues = NULL) = 0;

    virtual void ReserveConfigFile(const string& filename) = 0; ///< Сохранение резервной копии заданного конфигурационного файла
    virtual void RestoreConfigFile(const string& filename) = 0; ///< Восстановление резервной копии заданного конфигурационного файла
    virtual void RemoveConfigFileReserve(const string& filename) = 0;  ///< Удалении резервной копии заданного конфигурационного файла

    virtual void RemoveALDMarkersFromConfigFile(const string& filename) = 0; ///< Удаление комментариев ALD из заданного конфигурационного файла

    /*! \brief Установка значения параметра конфигурационного файла ALD (ald.conf) по имени
     * @param[in] name Имя параметра
     * @param[in] value Значение параметра
     */
    virtual bool UpdateConfigFileParam(const string &name, const string &value) = 0;

    /*! \brief Получение значения подстановочной переменной по имени
     * @param[in] p Имя переменной
     */
    virtual string TemplateValue(const string &p) = 0;

    /*! \brief Проверка существования подстановочной переменной по имени
     * @param[in] p Имя переменной
     * @return True в случае существования
     */
    virtual bool TemplateValueExists(const string &p) = 0;

    /*! \brief Установка значения подстановочной переменной по имени
     * @param[in] p Имя переменной
     * @param[in] v Значение переменной
     */
    virtual void SetTemplateValue(const string &p, const string &v) = 0;

    /*! \brief Сохранение статуса ядра ALD для заданного режима
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * \details В файле статуса сохраняется содержимое внутренних параметров ядра
     */
    virtual void SaveConfigStatus(const string& mode) = 0;

    /*! \brief Проверка существования статуса ядра ALD для заданного режима
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @return True в случае существования
     */
    virtual bool IsConfigStatusExist(const string& mode) = 0;

    /*! \brief Удаение статуса ядра ALD для заданного режима
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     */
    virtual void RemoveConfigStatus(const string& mode) = 0;

    /*! \brief Загрузка статуса ядра ALD для заданного режима
     * @param[in] mode Режим (например ALD_MODE_CLIENT или ALD_MODE_SERVER)
     * @param[out] status Таблица сохраненных в статусе параметров ядра ALD
     */
    virtual void LoadConfigStatus(const string& mode, ald_string_map& status) = 0;

    virtual void SaveFileServerStatus() = 0; ///< Сохранение статуса файлового сервера
    virtual string GetFileServerStatus() = 0; ///< Загрузка статуса файлового сервера, возвращается тип файлового сервера
    virtual void RemoveFileServerStatus() = 0; ///< Удаление статуса файлового сервера

    virtual ~IALDConfig(){}  ///< Деструктор
};

typedef shared_ptr<IALDConfig> CALDConfigPtr;  ///< Разделяемый указатель на интерфейс конфигурирования Astra Linux Directory

/// @}

} // ALD

#endif // ALD_ICONFIG



