/*! \file ald-rpc.h
 * \brief Объявление RPC интерфейса Astra Linux Directory
 * \ingroup IALDRpcWrapper_group
 */

#ifndef ALD_RPC
#define ALD_RPC

/*! \name Константы типов RPC сообщений
 * \ingroup IALDRpcWrapper_group */
/*! @{ */
#define ARPCM_ERROR       0x0
#define ARPCM_CALL        0x10
#define ARPCM_ECALL       0x11
#define ARPCM_RESULT      0x20
#define ARPCM_ERESULT     0x21
#define ARPCM_LOCAL_REQ   0x30
#define ARPCM_LOCAL_RES   0x31
#define ARPCM_BROAD_MSG   0x40
#define ARPCM_BROAD_RES   0x41
/*! @} */

/*! Имя службы ALD
* \ingroup IALDRpcWrapper_group */
#define ALDRPCSERVICE "aldd"

/*! Сигнатура (ALDRPCMAGIC)
* \ingroup IALDRpcWrapper_group */
#define ALDRPCMAGIC "ALDRPC01"

/*! Версия (ALDRPCVERSION)
* \ingroup IALDRpcWrapper_group */
#define ALDRPCVERSION  100

/*! Структура заголовка RPC запроса
* \ingroup IALDRpcWrapper_group */
struct ald_rpc_header {
    /*! MAGIG */
    char            magic[8];
    /*! Версия */
    unsigned short  version;
    /*! Тип сообщения */
    unsigned short  message;
    /*! Размер данных */
    unsigned long   size;
};

#ifdef __cplusplus

#include "common.h"
#include "ALDCore.h"
#include "gettext.h"

using namespace std;

namespace ALD
{

/*! \defgroup IALDRpcWrapper_group Интерфейс RPC администрирования (IALDRpcWrapper)
 * \brief RPC интерфейс Astra Linux Directory
 * @{
 */

/*! Имя RPC интерфейса */
#define ALD_RPC_ITF_NAME "ald-rpc"

/*! Константа размера заголовка RPC запроса */
const size_t ALDRPCHDRSIZE = sizeof(ald_rpc_header);

/*! Статус состояния RPC запроса */
enum alRPCStatus {alrsINI, alrsOK, alrsWAIT, alrsERR};

/*! \struct ald_rpc_request
 * \brief Структура RPC запроса
 * \details Инкапсулирует заголовок RPC запроса, аргументы, статус и результат исполнения
 * \ingroup IALDRpcWrapper_group
 */
struct ald_rpc_request {
    /*! Заголовок RPC запроса */
    ald_rpc_header header;
    /*! Имя команды */
    string name;
    /*! Таблица аргументов */
    ald_string_multimap args;
    /*! Таблица результатов */
    ald_string_multimap result;
    /*! Статус запроса */
    alRPCStatus status;
    /*! Сообщение об ошибке */
    string error;

    /*! Проверка успешного статуса запроса */
    inline bool good() const {return alrsOK == status;}

    /*! \brief Установка аргумента
     * @param[in] arg Имя аргумента
     * @param[in] val Значение
     */
    inline void addArg(const string &arg, const string &val){args.insert(make_pair(arg, val));}

    /*! \brief Установка результата
     * @param[in] res Имя результата
     * @param[in] val Значение
     */
    inline void addRes(const string &res, const string &val){result.insert(make_pair(res, val));}

    /*! Установка ошибки */
    inline void setError(const string &err){error = err;status = alrsERR;}

    /*! Получение значения аргумента по имени (при отсутствии - пустая строка) */
    inline string argByName(const string& key){
        ald_string_multimap::const_iterator it = args.find(key);
        if(it != args.end())
            return it->second;
        else
            return "";
    }

    /*! Получение значения результата по имени (при отсутствии - пустая строка) */
    inline string resByName(const string& key){
        ald_string_multimap::const_iterator it = result.find(key);
        if(it != result.end())
            return it->second;
        else
            return "";
    }

    /*! Проверка существования аргумента по имени */
    inline bool argExists(const string& key){
        ald_string_multimap::const_iterator it = args.find(key);
        return (it != args.end());
    }

    /*! Проверка существования результата по имени */
    inline bool resExists(const string& key){
        ald_string_multimap::const_iterator it = result.find(key);
        return (it != result.end());
    }

    /*! Очистка таблиц аргументов и результатов */
    inline void clear(){args.clear(); result.clear();}
};

/*! \class IALDRpcConnection
 * \brief Абстрактный класс-интерфейс RPC соединения с сервером ALD
 * \details Предоставляет интерфейс выполнения RPC запросов к серверу ALD
 */
class IALDRpcConnection
{
public:
    /*! Текущий способ аутентификации */
    virtual ALDAuthType AuthType() = 0;
    /*! Текущее значение таймаута */
    virtual int getTimeout() const = 0;
    /*! Установка значения таймаута */
    virtual void setTimeout(int timeout) = 0;

    /*! \brief Установка соединения
     * @param[in] server FQDN адрес сервера ALD
     * @param[in] at Способ аутентификации ALDAuthType
     * @return Успешность выполнения операции
     */
    virtual bool Connect(const string &server, ALDAuthType at) = 0;
    /*! Проверка установлено ли соединение */
    virtual bool IsConnected() = 0;
    /*! Завершение соединения с сервером */
    virtual void Disconnect() = 0;

    /*! Выполнение RPC запроса к серверу ALD */
    virtual void Call(ald_rpc_request& req) = 0;
    /*! Выполнение RPC запроса к серверу ALD с получением кода возврата */
    virtual alRPCStatus CallEx(ald_rpc_request& req) = 0;

    /*! Деструктор */
    virtual ~IALDRpcConnection(){}
};

/*! Разделяемый указатель на интерфейс RPC соединения с сервером */
typedef shared_ptr<IALDRpcConnection> CALDRpcConnPtr;

/*! \class IALDRpcWrapper
 * \brief Абстрактный класс-интерфейс RPC взаимодействия с сервером
 * \details Предоставляет интерфейс установки RPC соединения с сервером
 */
class IALDRpcWrapper: public CALDInterface
{
public:
    /*! Конструктор */
    IALDRpcWrapper(IALDCore& Core):CALDInterface(ALD_RPC_ITF_NAME, Core){}
    /*! Отправка широковещательного сообщения msg */
    virtual size_t SendBroadcastMessage(const string& msg, ald_string_list *responses, int timeout = 3) = 0;
    /*! Создание экземпляра RPC соединения с сервером ALD CALDRpcConnPtr */
    virtual CALDRpcConnPtr getConnection() = 0;
};

/*! Разделяемый указатель на интерфейс RPC взаимодействия с сервером IALDRpcWrapper */
typedef shared_ptr<IALDRpcWrapper> CALDRpcWrapperPtr;

} /* ALD */

#endif /* ALD_RPC */

#endif /* __cplusplus */

/*! @} */

