/*! \file kadm5-wrapper.h
 * \brief Объявление интерфейса взаимодействия с Kerberos
 * \ingroup IALDKadm5Wrapper_group
 */

#ifndef ALD_KADM5_WRAPPER
#define ALD_KADM5_WRAPPER

#include "common.h"
#include "ALDCore.h"
#include <list>
#include <memory>

using namespace std;

namespace ALD
{
/*! \defgroup IALDKadm5Wrapper_group Интерфейс взаимодействия с Kerberos (IALDKadm5Wrapper)
 * \brief Интерфейс взаимодействия с Kerberos
 * @{
 */

/// Имя интерфейса Kerberos
#define KADM5_ITF_NAME "kadmin"

/// Версия 1 протокола Kerberos
#define KRB5_ITF_VERSION_1	1
/// Версия 2 протокола Kerberos
#define KRB5_ITF_VERSION_2	2

/// Имя переменной окружения KRB5CCNAME, содержащей имя кеша аутентификационных данных по умочанию
#define KRB5CCNAME "KRB5CCNAME"

/*! \class CALDKrbPolicy
 * \brief Класс политики паролей Kerberos
 */
class CALDKrbPolicy
{
public:
    string  name;                   ///< Имя политики
    unsigned int itf_version;       ///< Версия протокола Kerberos
    unsigned int min_life;          ///< Минимальное время действия пароля
    unsigned int max_life;          ///< Максимальное время действия пароля
    unsigned int min_length;        ///< Минимальная длина
    unsigned int min_classes;       ///< Минимальное количество классов символов
    unsigned int history_num;       ///< Размер истории паролей
    unsigned int refcnt;            ///< Счетчик использования политики
    /* KRB5_ITF_VERSION_2 */
    unsigned int max_fail;          ///< Максимальное количество неверных вводов пароля до блокировки
    unsigned int failcnt_interval;  ///< Время между попытками ввода при неверном вводе
    unsigned int lockout_duration;  ///< Длительность блокировки
    CALDKrbPolicy()                 ///< Конструктор
    {
        itf_version = KRB5_ITF_VERSION_2;
        max_life = 0;
        min_life = 0;
        min_length = 1;
        min_classes = 1;
        history_num = 0;
        refcnt = 0;
        max_fail = 0;
        failcnt_interval = 0;
        lockout_duration = 0;
    }

    /*! \brief Копирование свойств с другой политики
     * @param[in] policy Экземпляр политики
     */
    void Assign(CALDKrbPolicy &policy)
    {
        name = policy.name;
        max_life = policy.max_life;
        min_life = policy.min_life;
        min_length = policy.min_length;
        min_classes = policy.min_classes;
        history_num = policy.history_num;
        refcnt = policy.refcnt;
        max_fail = policy.max_fail;
        failcnt_interval = policy.failcnt_interval;
        lockout_duration = policy.lockout_duration;
    }
    virtual ~CALDKrbPolicy(){} ///< Деструтор
};

/*! \name Элементы маски модификации политики */
///@{
/// Флаг установки максимального времени действия паролей
#define ALD_KRB_SET_PW_MAX_LIFE     0x01
/// Флаг установки минимального времени действия паролей
#define ALD_KRB_SET_PW_MIN_LIFE     0x02
/// Флаг установки минимальной длины паролей
#define ALD_KRB_SET_PW_MIN_LENGTH   0x04
/// Флаг установки минимального количества классов символов в пароле
#define ALD_KRB_SET_PW_MIN_CLASSES  0x08
/// Флаг установки размера истории паролей
#define ALD_KRB_SET_PW_HISTORY_NUM  0x10

/// Флаг установки максимального количества неверных вводов пароля до блокировки
#define ALD_KRB_SET_PW_MAX_FAIL	    0x20
/// Флаг установки времени между попытками ввода при неверном вводе
#define ALD_KRB_SET_PW_FAIL_CNT_INT 0x40
/// Флаг установки длительности блокировки
#define ALD_KRB_SET_PW_LCK_OUT_DUR  0x80
///@}

/// Вектор флагов установки всех свойств политики паролей
#define ALD_KRB_SET_PW_ALL	    ALD_KRB_SET_PW_MAX_LIFE | ALD_KRB_SET_PW_MIN_LIFE | ALD_KRB_SET_PW_MIN_LENGTH \
                                | ALD_KRB_SET_PW_MIN_CLASSES | ALD_KRB_SET_PW_HISTORY_NUM | ALD_KRB_SET_PW_MAX_FAIL \
                                | ALD_KRB_SET_PW_FAIL_CNT_INT | ALD_KRB_SET_PW_LCK_OUT_DUR

typedef shared_ptr<CALDKrbPolicy> CALDKrbPolicyPtr;  ///< Разделяемый указатель на политику паролей Kerberos CALDKrbPolicy

/*! \class CALDKrbPrincipal
 * \brief Класс принципала Kerberos
 */
class CALDKrbPrincipal
{
public:
    string        name;                 ///< Имя принципала
    unsigned int  itf_version;          ///< Версия протокола Kerberos
    string        policy;               ///< Имя политики паролей
    time_t        princ_expire_time;    ///< Максимальное время действия учетной записи
    time_t        last_pwd_change;      ///< Время последней смены пароля
    time_t        pw_expiration;        ///< Время истечения действия пароля
    time_t        mod_date;             ///< Время модификации
    time_t        last_success;         ///< Время последней успешной попытки аутентификации
    time_t        last_failed;          ///< Время последней неуспешной попытки аутентификации
    unsigned int  fail_auth_count;      ///< Количество неуспешных попыток аутентификации
    unsigned int  attributes;           ///< Вектор атрибутов
    unsigned int  max_life;             ///< Максимальное время действия билета
    unsigned int  max_renewable_life;   ///< Максимальное возобновляемое время действия билета
    ald_string_map strings;             ///< Строковые атрибуты
    CALDKrbPrincipal()  ///< Конструктор
    {
        itf_version = KRB5_ITF_VERSION_2;
        princ_expire_time = 0;
        last_pwd_change = 0;
        pw_expiration = 0;
        mod_date = 0;
        last_success = 0;
        last_failed = 0;
        fail_auth_count = 0;
        attributes = 0;
        max_life = 0;
        max_renewable_life = 0;
    }
    virtual ~CALDKrbPrincipal(){}   ///< Деструктор
};

/*! \name Элементы маски модификации принципала */
///@{
/// = KRB5_KDB_DISALLOW_ALL_TIX
#define ALD_KRB_DISALLOW_ALL_TIX    0x00000040
/// = KRB5_KDB_REQUIRES_PWCHANGE
#define ALD_KRB_REQUIRES_PWCHANGE   0x00000200
/// = KRB5_KDB_DISALLOW_TGT_BASED
#define ALD_KRB_DISALLOW_TGT_BASED  0x00000004

/// Флаг установки политики паролей
#define ALD_KRB_SET_POLICY              0x000800
/// Флаг генерации случайного ключа
#define ALD_KRB_SET_RANDKEY             0x001000
/// Флаг смены пароля
#define ALD_KRB_SET_PWCHANGE            0x002000
/// Флаг установки запрещения выдачи билетов
#define ALD_KRB_SET_DISALLOWTIX         0x004000
/// Флаг установки максимального времени действия билета
#define ALD_KRB_SET_MAXLIFE             0x008000
/// Флаг установки максимального времени возобновляемого действия билета
#define ALD_KRB_SET_MAXRLIFE            0x010000
/// Флаг установки максимального количества неверных попыток аутентификации
#define ALD_KRB_CLR_FAILCNT             0x020000
/// Флаг установки запрета выдачи билета на основе TGT
#define ALD_KRB_SET_DISALLOWTGTBASED    0x040000
///@}

/*! \name Константы строковых атрибутов принципалов */
///@{
#define ALD_KRB_STR_ACCOUNT_TYPE "ald_account"
#define ALD_KRB_STR_ACCOUNT_USER "user"
#define ALD_KRB_STR_ACCOUNT_SERVICE "service"
#define ALD_KRB_STR_ACCOUNT_INTERNAL "internal"
///@{

typedef shared_ptr<CALDKrbPrincipal> CALDKrbPrincipalPtr;   ///< Разделяемый указатель на принципал Kerberos CALDKrbPrincipal

/*! \class IALDKadm5Connection
 * \brief Абстрактный класс-интерфейс соединения с сервером Kerberos
 * \details Предоставляет интерфейс выполнения запросов к серверу Kerberos
 */
class IALDKadm5Connection
{
public:
    virtual unsigned int GetItfVersion() = 0;   ///< Версия протокола Kerberos

    // Connection
    /*! \brief Установка соединения с помощью пароля
     * @param[in] user Имя пользователя
     * @param[in] pwd Пароль
     * @param[in] kadmin Флаг необходимости использования интерфейса администрирования
     * @param[in] savecred Флаг необходимости сохранения кеша аутентификационных данных
     */
    virtual void ConnectPassword(const string& user, const string& pwd, bool kadmin, bool savecred = false) = 0;

    /*! \brief Установка соединения с помощью файла ключей
     * @param[in] user Имя пользователя
     * @param[in] keytab Путь к файлу ключей
     * @param[in] kadmin Флаг необходимости использования интерфейса администрирования
     * @param[in] savecred Флаг необходимости сохранения кеша аутентификационных данных
     */
    virtual void ConnectKeytab(const string& user, const string& keytab, bool kadmin, bool savecred = false) = 0;

    /*! \brief Установка соединения с помощью кеша аутентификационных данных
     * @param[in] cache Имя кеша аутентификационных данных
     * @param[in] kadmin Флаг необходимости использования интерфейса администрирования
     */
    virtual string ConnectCreds(const string& cache, bool kadmin) = 0;

    virtual bool IsConnected() = 0;                         ///< Проверка установлено ли соединение

    /*! \brief Завершение соединения с сервером
     * @param[in] keep_cred Флаг отмены удаления кеша аутентификационных данных
     */
    virtual void Disconnect(bool keep_cred = false) = 0;

    // Policies
    /*! \brief Получение списка политик паролей по шаблону
     * @param[in] expr Шаблон имени политики паролей
     * @param[out] pollist Список имен политик паролей
     */
    virtual void Policies(const string& expr, ald_string_list& pollist) = 0;

    /*! \brief Получение политики паролей по имени
     * @param[in] polname Имя политики паролей
     */
    virtual CALDKrbPolicyPtr PolicyGet(const string& polname) = 0;

    /*! \brief Создание политики паролей
     * @param[in] Policy Экземпляр класса политики паролей
     * @param[in] setmask Вектор флагов устанавливаемых полей
     */
    virtual void PolicyAdd(CALDKrbPolicyPtr Policy, unsigned int setmask) = 0;

    /*! \brief Модификация политики паролей
     * @param[in] Policy Экземпляр класса политики паролей
     * @param[in] setmask Вектор флагов устанавливаемых полей
     */
    virtual void PolicyMod(CALDKrbPolicyPtr Policy, unsigned int setmask) = 0;

    /*! \brief Удаление политики паролей по имени
     * @param[in] policy Имя политики паролей
     */
    virtual void PolicyDel(const string& policy) = 0;

    // Principals
    /*! \brief Получение списка принципалов по шаблону
     * @param[in] expr Шаблон имени принципала
     * @param[out] princlist Список имен принципалов
     */
    virtual void Principals(const string& expr, ald_string_list& princlist) = 0;

    /*! \brief Получение принципала по имени
     * @param[in] princname Имя принципала
     */
    virtual CALDKrbPrincipalPtr PrincipalGet(const string& princname) = 0;

    /*! \brief Создание принципала
     * @param[in] Principal Экземпляр принципала
     * @param[in] pwd Пароль
     * @param[in] setmask Вектор флагов устанавливаемых полей
     */
    virtual void PrincipalAdd(CALDKrbPrincipalPtr Principal, const string& pwd, unsigned int setmask) = 0;

    /*! \brief Модификация принципала
     * @param[in] Principal Экземпляр принципала
     * @param[in] setmask Вектор флагов устанавливаемых полей
     */
    virtual void PrincipalMod(CALDKrbPrincipalPtr Principal, unsigned int setmask) = 0;

    /*! \brief Удаление принципала
     * @param[in] Principal Экземпляр принципала
     */
    virtual void PrincipalDel(CALDKrbPrincipalPtr Principal) = 0;

    /*! \brief Получение строковых атрибутов принципала
     * @param[in] Principal Экземпляр принципала
     */
    virtual void PrincipalGetStrings(CALDKrbPrincipalPtr Principal) = 0;

    /*! \brief Устанвока/удаление строкового атрибута принципала
     * @param[in] Principal Экземпляр принципала
     * @param[in] key Ключ
     * @param[in] val Значение
     */
    virtual void PrincipalSetString(CALDKrbPrincipalPtr Principal, const string &key, const string &val) = 0;

    /*! \brief Смена пароля принципала
     * @param[in] Principal Экземпляр принципала
     * @param[in] pwd Пароль
     */
    virtual void PrincipalChangePwd(CALDKrbPrincipalPtr Principal, const string& pwd) = 0;

    // Keytab
    /*! \brief Проверка ключей принципала в файле ключей
     * @param[in] keytab Путь к файлу ключей
     * @param[in] princname Имя принципала
     * @return Наличие корректных ключей в файле
     */
    virtual bool KeytabCheckPrincipal(const string& keytab, const string& princname) = 0;

    /*! \brief Добавление ключей принципала в файл ключей
     * @param[in] keytab Путь к файлу ключей
     * @param[in] princname Имя принципала
     */
    virtual void KeytabAddPrincipal(const string& keytab, const string& princname) = 0;

    /*! \brief Удаление ключей принципала из файла ключей
     * @param[in] keytab Путь к файлу ключей
     * @param[in] princname Имя принципала
     */
    virtual void KeytabRmPrincipal(const string& keytab, const string& princname) = 0;

    /*! \brief Получение списка принципалов из файла ключей
     * @param[in] keytab Путь к файлу ключей
     * @param[in] princlist Список имен принципалов
     */
    virtual void KeytabPrincipals(const string& keytab, ald_string_list& princlist) = 0;

    // States and properties
    virtual string GetDefaultRealm() = 0;   ///< Kerberos REALM по умолчанию
    virtual long GetLastError() = 0;        ///< Получение кода последней ошибки
    virtual bool IfReconnectNeeded() = 0;   ///< Проверка нужна ли переустановка соединения
    virtual string GetUserId() = 0;         ///< Текущий аутентифицированный пользователь

    virtual ~IALDKadm5Connection(){} ///< Деструктор
};

typedef shared_ptr<IALDKadm5Connection> CALDKadm5ConnPtr; ///< Разделяемый указатель на интерфейс соединения с сервером Kerberos


/*! \class IALDKadm5Wrapper
 * \brief Абстрактный класс-интерфейс взаимодействия с Kerberos
 * \details Предоставляет интерфейс установки соединения с сервером Kerberos
 */
class IALDKadm5Wrapper: public CALDInterface
{
public:
    IALDKadm5Wrapper(IALDCore& Core):CALDInterface(KADM5_ITF_NAME, Core){} ///< Конструктор
    virtual CALDKadm5ConnPtr getConnection() = 0;    ///< Создание экземпляра соединения с сервером Kerberos CALDKadm5ConnPtr
};

typedef shared_ptr<IALDKadm5Wrapper> CALDKadm5WrapperPtr; ///< Разделяемый указатель на интерфейс взаимодействия с Kerberos IALDKadm5Wrapper

/// Имя мастер-принципала Kerberos
#define ADM_KRB_USER_NAME         "K/M"

/// @}
} // ALD

#endif  // ALD_KADM5_WRAPPER
