/*! \file ldap-wrapper.h Объявление интерфейса взаимодействия с LDAP
 * \brief  Объявление интерфейса взаимодействия с LDAP
 * \ingroup IALDLdapWrapper_group
 */

#ifndef ALD_LDAP_WRAPPER
#define ALD_LDAP_WRAPPER

#include "common.h"
#include "ALDCore.h"
#include <map>
#include <memory>

using namespace std;

namespace ALD
{
/*! \defgroup IALDLdapWrapper_group Интерфейс взаимодействия с LDAP (IALDLdapWrapper)
 * \brief Интерфейс взаимодействия с LDAP
 * @{
 */

/// Имя LDAP интерфейса
#define LDAP_ITF_NAME "ldap"

/// Константа для аргумента nullable, определяющая аргумент с разрешенным значением NULL
#define LDAP_ATTR_NULLABLE  true

/// Уровни просмотра LDAP дерева
enum alSCOPE {alsBASE, alsONELEVEL, alsSUBTREE, alsCHILDREN};

/// Операции LDAP (префиксы при модификации записи LDAP)
enum alOPERATION {
    aloADD = '+',          ///< Добавление заданного значения
    aloREPLACE = '=',      ///< Установка заданного значения
    aloDELVAL = '-',       ///< Удаление заданного значения
    aloDELETE = '!'        ///< Удаление всех значений атрибута (удаление атрибута)
};

/*! \class IALDLdapEntity
 * \brief Абстрактный класс-интерфейс LDAP записи
 * \details Инкапсулирует уникальное имя и атрибуты записи LDAP.
 * Предоставляет интерфейс создания, модификации, удаления и переименования.
 */
class IALDLdapEntity
{
public:

    string DN;                  ///< Уникальное имя
    ald_string_multimap Attrs;  ///< Таблица значений атрибутов (имя, значение)

    /*! \brief Получение значения атрибута по имени
     * @param[in] Attr Имя атрибута
     * @param[in] nullable Признак возможного NULL значения
     * @return Значение атрибута
     */
    virtual string ValByAttr(const string& Attr, bool nullable = false) = 0;

    /*! \brief Получение списка значений атрибута по имени
     * @param[in] Attr Имя атрибута
     * @param[out] vals Результирующий список значений
     * @param[in] nullable Признак возможного NULL значения
     * @return Количество значений
     */
    virtual size_t ValsByAttr(const string& Attr, ald_string_list &vals, bool nullable = false) = 0;


    virtual void Clear() = 0;       ///< Очистка экземпляра класса
    virtual bool IsValid() = 0;     ///< Проверка валидности (не пустота #DN)
    virtual void Create() = 0;      ///< Создание
    virtual void Delete() = 0;      ///< Удаление


    /*! \brief Модификация записи
     * @param[in] modattrs Таблица модифицируемых значений атрибутов.\n
     * Имя атрибута в каждой паре должно предваряться префиксом при
     * модификации записи LDAP из набора #alOPERATION. Например:
     * @code
     * if(!newdesc.empty())
     *   newattrs.insert(ald_string_pair(string("=")+ALD_LDAP_GROUP_DESC, newdesc));
     * else if(!desc().empty())
     *   newattrs.insert(ald_string_pair(string("!")+ALD_LDAP_GROUP_DESC, desc()));
     * @endcode
     *
     * @param[in] inplace Признак модификации "по месту" (без записи в LDAP)
     */
    virtual void Update(ald_string_multimap &modattrs, bool inplace = false) = 0;

    /*! \brief Переименование записи
     * @param[in] newrdn Новый путь в древе LDAP
     */
    virtual void Rename(const string &newrdn) = 0;

    virtual string creator() = 0;       ///< dn создателя
    virtual string modifier() = 0;      ///< dn модификатора
    virtual string created() = 0;       ///< время создания (GeneralizedTime = LDAP UTCTime syntax - 1.3.6.1.4.1.1466.115.121.1.53)
    virtual string modified() = 0;      ///< время модификации (GeneralizedTime = LDAP UTCTime syntax - 1.3.6.1.4.1.1466.115.121.1.53)

    virtual ~IALDLdapEntity(){}     ///< Деструктор
};

typedef shared_ptr<IALDLdapEntity> CALDLdapEntPtr;  ///< Разделяемый указатель на интерфейс записи LDAP IALDLdapEntity

typedef list<CALDLdapEntPtr> ald_ldap_ent_list; ///< Список разделяемых указателей на интерфейс записи LDAP IALDLdapEntity

/*! \class IALDLdapQuery
 * \brief Абстрактный класс-интерфейс результирующего набора записей LDAP
 * \details Предоставляет интерфейс перебора записей результата запроса к LDAP аналогично
 * курсору SQL.
 */
class IALDLdapQuery
{
public:
    virtual void First() = 0;       ///< Переход к первой записи набора

    /*! \brief Получение записи
     * @param[in] Entity Ссылка на разделяемый указатель на интерфейс записи LDAP IALDLdapEntity
     * @return Успешность выполнения операции
     */
    virtual bool Fetch(CALDLdapEntPtr &Entity) = 0;

    virtual int Count() = 0;        ///< Количество записей в наборе

    virtual ~IALDLdapQuery(){}     ///< Деструктор
};

typedef shared_ptr<IALDLdapQuery> CALDLdapQueryPtr; ///< Разделяемый указатель на интерфейс результирующего набора записей LDAP IALDLdapQuery

/*! \class IALDLdapConnection
 * \brief Абстрактный класс-интерфейс соединения с сервером LDAP
 * \details Предоставляет интерфейс выполнения запросов к серверу LDAP
 */
class IALDLdapConnection
{
public:
    virtual ~IALDLdapConnection(){} ///< Деструктор

    virtual ALDAuthType AuthType() = 0;     ///< Текущий способ аутентификации

    /*! \brief Установка соединения
     * @param[in] server Строка соединения с LDAP сервером (URI)
     * @param[in] at Способ аутентификации ALDAuthType
     * @return Успешность выполнения операции
     */
    virtual bool Connect(const string &server, ALDAuthType at) = 0;
    virtual bool IsConnected() = 0;         ///< Проверка установлено ли соединение
    virtual void Disconnect() = 0;          ///< Завершение соединения с сервером

    // utils
    virtual long GetLastError() = 0;    ///< Получение кода последней ошибки
    virtual bool IfReconnectNeeded() = 0;   ///< Проверка нужна ли переустановка соединения
    virtual string LocalToUtf8(const string &strLocal) = 0;     ///< Перевод строки из локальной кодировки в UTF8
    virtual string Utf8ToLocal(const string &strUtf8) = 0;      ///< Перевод строки из UTF8 в локальную кодировку
    virtual void AdjustGecosType() = 0;         ///< Изменение типа поля gecos для поддержки русского языка

    // operations
    virtual CALDLdapEntPtr AllocEntity() = 0;   ///< Создание экземпляра связанного с соединением класса LDAP записи

    /*! \brief Запрос к LDAP серверу
     * \details Выполняет поиск в дереве LDAP аналогично функции ldap_search или утилиты ldapsearch
     * @param[in] basedn Начальная точка поиска
     * @param[in] filter Запрос (фильтр)
     * @param[in] attrs Список выбираемых атрибутов
     * @param[in] scope Уровни просмотра LDAP дерева alSCOPE
     * @return Разделяемый указатель на интерфейс результирующего набора записей LDAP IALDLdapQuery
     */
    virtual CALDLdapQueryPtr Search(const string &basedn, const string &filter,
                       ald_string_list &attrs, alSCOPE scope = alsSUBTREE) = 0;

    /*! \brief Удаление атрибута записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attr Имя атрибута
     */
    virtual void DeleteAttr(const string &dn, const string &attr) = 0;

    /*! \brief Модификация значения атрибута записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attr Имя атрибута
     * @param[in] val Значение атрибута
     * @param[in] op Операция LDAP alOPERATION
     */
    virtual void SetAttrValue(const string &dn, const string &attr, const string &val,
                              alOPERATION op = aloREPLACE) = 0;

    /*! \brief Модификация значений атрибута записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attr Имя атрибута
     * @param[in] vals Список значений атрибута
     * @param[in] op Операция LDAP alOPERATION
     */
    virtual void SetAttrValues(const string &dn, const string &attr, ald_string_list &vals,
                               alOPERATION op = aloREPLACE) = 0;

    /*! \brief Модификация записи
     * @param[in] dn Уникальное имя записи
     * @param[in] modattrs Таблица модифицируемых значений атрибутов.\n
     * Имя атрибута в каждой паре должно предваряться префиксом при
     * модификации записи LDAP из набора #alOPERATION. Например:
     * @code
     * if(!newdesc.empty())
     *   newattrs.insert(ald_string_pair(string("=")+ALD_LDAP_GROUP_DESC, newdesc));
     * else if(!desc().empty())
     *   newattrs.insert(ald_string_pair(string("!")+ALD_LDAP_GROUP_DESC, desc()));
     * @endcode
     */
    virtual void UpdateDN(const string &dn, ald_string_multimap &modattrs) = 0;

    virtual void DeleteDN(const string &dn) = 0;      ///< Удаление записи по уникальному имени dn

    /*! \brief Создание записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attrs Таблица значений атрибутов. Например:
     * @code
     * attrs.insert(ald_string_pair(ALD_LDAP_GROUP_DESC, newdesc));
     * @endcode
     */
    virtual void AddDN(const string &dn, ald_string_multimap &attrs) = 0;

    /*! \brief Переименование записи
     * @param[in] dn Уникальное имя записи
     * @param[in] newrdn Новый путь в древе LDAP
     */
    virtual void RenameDN(const string &dn, const string &newrdn) = 0;

    /*! \brief Модификация двоичного значения атрибута записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attr Имя атрибута
     * @param[in] pval Указатель на значение атрибута
     * @param[in] len Размер значение атрибута в байтах
     * @param[in] op Операция LDAP alOPERATION
     */
    virtual void SetAttrBinaryValue(const string &dn, const string &attr,
                                    const char* pval, unsigned long len,
                                    alOPERATION op = aloREPLACE) = 0;

    /*! \brief Получение двоичного значения атрибута записи
     * @param[in] dn Уникальное имя записи
     * @param[in] attr Имя атрибута
     * @param[out] len Размер значение атрибута в байтах
     * @return Указатель на значение атрибута - должен быть освобожден с помощью free()
     */
    virtual void* GetAttrBinaryValue(const string &dn, const string &attr,
                                    unsigned long &len) = 0;

    // maintaining
    virtual void EnsureOU(const string &dn) = 0;            ///< Автоматическое создание пути классов типа organizationalUnit для заданного уникального имени dn

    virtual void EnumerateIndexes(ald_string_list &indexes) = 0; ///< Получение списка индексов
    virtual void AddDBIndex(const string &idx) = 0;         ///< Добавление индекса idx
    virtual void DeleteDBIndex(const string &idx) = 0;      ///< Удаление индекса idx

    /*! \brief Получение списка правил доступа к LDAP
     * @param[out] rules Список правил доступа
     */
    virtual void EnumerateDBAccessRules(ald_string_list &rules) = 0;

    /// Определение константы вставки правила доступа в начало
    #define ALD_LDAP_ACCESS_POS_FIRST     0
    /// Определение константы вставки правила доступа в конец
    #define ALD_LDAP_ACCESS_POS_LAST      -1

    /*! \brief Добавление правила доступа
     * @param[in] idx Номер позиции вставки (могут быть #ALD_LDAP_ACCESS_POS_FIRST и #ALD_LDAP_ACCESS_POS_LAST)
     * @param[in] rule Правило доступа
     */
    virtual void AddDBAccessRule(int idx, const string &rule) = 0;

    virtual void DeleteDBAccessRule(int idx) = 0;   ///< Удаление правила доступа по номеру idx

    virtual void EnumerateLdapSchemas(ald_string_list &schemas) = 0;    ///< Получение списка зарегистрированных схем
    virtual bool IsLdapSchemaRegistered(const string &schema) = 0;      ///< Проверка зарегистрированна ли схема

    virtual void EnumerateModules(ald_string_list &modules) = 0;    ///< Получение списка загруженных модулей

    virtual void EnumerateDatabases(ald_string_map &dbs) = 0;       ///< Получение списка объявленных баз данных

    virtual void EnumerateDBUniqueness(ald_string_list &uniqs) = 0;   ///< Получение списка ограничений уникальности
    virtual void AddDBUniqueness(const string &uniq) = 0;           ///< Добавление ограничения уникальности
    virtual void DeleteDBUniqueness(const string &uniq) = 0;        ///< Удаление ограничения уникальности

    virtual void EnumerateDBLimits(ald_string_map &limits) = 0;      ///< Получение списка лимитов
    virtual void AddDBLimit(const string &who, const string &limit) = 0; ///< Добавление лимита
    virtual void DeleteDBLimit(const string &who) = 0;            ///< Удаление лимита

    //replication
    virtual void EnumerateSyncRepls(ald_string_list &syncrepls) = 0;    ///< Получение списка узлов репликации SyncRepl (модуль syncprov)
    virtual void AddSyncRepl(string rid, string provider) = 0;          ///< Добавление узла репликации SyncRepl
    virtual void DeleteSyncRepl(string rid, string provider) = 0;       ///< Удаление узла репликации SyncRepl

    //referral
    virtual void EnumerateLdapProviders(ald_string_map &providers) = 0; ///< Получение списка провайдеров LDAP

    /*! \brief Добавление провайдера LDAP
     * @param[in] domain Домен LDAP
     * @param[in] uri Адрес сервера LDAP (URI)
     * @param[in] pwd Пароль
     */
    virtual void AddLdapProvider(const string &domain, string uri, const string &pwd) = 0;

    /*! \brief Получение проавйдера LDAP по домену
     * @param[in] domain Домен LDAP
     * @return Разделяемый указатель на интерфейс записи LDAP CALDLdapEntPtr
     */
    virtual CALDLdapEntPtr GetLdapProvider(const string &domain) = 0;

    virtual void UpdateLdapProvider(const string &domain, string uri) = 0;  ///< Изменения адреса сервера LDAP для домена LDAP
    virtual void DeleteLdapProvider(const string &domain) = 0;  ///< Удаление провайдера репликация для домена LDAP
};

typedef shared_ptr<IALDLdapConnection> CALDLdapConnPtr; ///< Разделяемый указатель на интерфейс соединения с сервером LDAP

/*! \class IALDLdapWrapper
 * \brief Абстрактный класс-интерфейс взаимодействия с LDAP
 * \details Предоставляет интерфейс установки соединения с сервером LDAP
 */
class IALDLdapWrapper: public CALDInterface
{
public:
    IALDLdapWrapper(IALDCore& Core):CALDInterface(LDAP_ITF_NAME, Core){} ///< Конструктор
    virtual CALDLdapConnPtr getConnection() = 0;    ///< Создание экземпляра соединения с сервером LDAP CALDLdapConnPtr
    virtual void RegisterLdapSchema(const string &schema) = 0;  ///< Регистрация заданной схемы на локальном сервере LDAP
    virtual string getDefaultURIs() = 0; ///< Список серверов LDAP по умолчанию
};

typedef shared_ptr<IALDLdapWrapper> CALDLdapWrapperPtr; ///< Разделяемый указатель на интерфейс взаимодействия с LDAP IALDLdapWrapper

/// @}

} // ALD

#endif  // ALD_LDAP_WRAPPER
